//
//
//
//
//
//                 program CAGeM 1.2.4 ("Computed-Aided Generation of Motion")
//                        ----------------------------------------------
//                         file MuPAD (version 2.5) - principal program
//                        
//
// copyright (C) 2003 Olivier VERLINDEN
//    Service de Mecanique rationnelle, Dynamique et Vibrations
//    Faculte Polytechnique de Mons
//    31, Bd Dolez, 7000 MONS (Belgium)
//    Olivier.Verlinden@fpms.ac.be
//
// This file is part of EasyDyn.
//
// EasyDyn is free software; you can redistribute it and/or modify it under the terms of the GNU General
// Public License as published by the Free Software Foundation; either version 2, or (at your option) 
// any later version.
//
// EasyDyn is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
// License for more details.
//
// You should have received a copy of the GNU General Public License along with EasyDyn; see the file
// COPYING.  If not, write to the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
// MA  02111-1307, USA.
//
//





// CAGeM procedure - definition of local and global variables.
cagem:=proc()
local time0, timeRD, timeKI1, timeKI2, timeSP1, timeSP2, timeCPP, timePLT;
local ssRD, ssKI1, ssKI2, ssKI, ssSP1, ssSP2, ssSP, ssCPP, ssPLT;
local timeTEXFR, ssTEXFR, timeTEXEN, ssTEXEN, timeTotal, ss, mm, seconds;
local filePLT, fileTEXFR, fileTEXEN, fileCPP;
local nbrbodymax, nbrdofmax, nbrdepmax, indexdof, indexbody, indexdep, index_relative;
local i, j, k, f, co, gen, temp, phiG, pi, pdi, pddi, nbr_relative;
begin
save SIMPLIFY, ADVANCED, ANIM, FORCES, STATIC, PLOT, LATEX_FR, LATEX_EN;
save title, repertoire, FichierUtil;
save mass, IG, gravity, T0G, T0Gd, R0G, R0Gd, MatRot, omega, omegad, vG, aG, nbrbody, nbrdof;
save  vGpartial, omegapartial ;



// Program initialization.
print(Unquoted,"===================================================================="):
print(Unquoted,"CAGeM v1.2.4"):
print(Unquoted,"(\"Computed-Aided Generation of Motion\")"):
print(Unquoted,"===================================================================="):
textinput("Complete name of the file to analyze (e.g. C:/4meca/student.mu) : ",FichierUtil):



// Procedure giving the homogeneous transformation matrix of the rotation about X-axis by an angle thetax.
Trotx:=proc(thetax)
   local Toi, M;
   begin
     M:=Dom::Matrix():
     Toi:=M(4,4,[[],[],[],[0,0,0,1]]):
     Toi[1,1]:= 1:
     Toi[1,2]:= 0:
     Toi[1,3]:= 0:
     Toi[1,4]:= 0:
     Toi[2,1]:= 0:
     Toi[2,2]:= cos(thetax):
     Toi[2,3]:= -sin(thetax):
     Toi[2,4]:= 0:
     Toi[3,1]:= 0:
     Toi[3,2]:= sin(thetax):
     Toi[3,3]:= cos(thetax):
     Toi
   end_proc:



// Procedure giving the homogeneous transformation matrix of the rotation about Y-axis by an angle thetay.
Troty:=proc(thetay)
   local Toi, M;
   begin
     M:=Dom::Matrix():
     Toi:=M(4,4,[[],[],[],[0,0,0,1]]):
     Toi[1,1]:= cos(thetay):
     Toi[1,2]:= 0:
     Toi[1,3]:= sin(thetay):
     Toi[1,4]:= 0:
     Toi[2,1]:= 0:
     Toi[2,2]:= 1:
     Toi[2,3]:= 0:
     Toi[2,4]:= 0:
     Toi[3,1]:= -sin(thetay):
     Toi[3,2]:= 0:
     Toi[3,3]:= cos(thetay):
     Toi
   end_proc:



// Procedure giving the homogeneous transformation matrix of the rotation about Z-axis by an angle thetaz.
Trotz:=proc(thetaz)
   local Toi, M;
   begin
     M:=Dom::Matrix():
     Toi:=M(4,4,[[],[],[],[0,0,0,1]]):
     Toi[1,1]:= cos(thetaz):
     Toi[1,2]:= -sin(thetaz):
     Toi[1,3]:= 0:
     Toi[1,4]:= 0:
     Toi[2,1]:= sin(thetaz):
     Toi[2,2]:= cos(thetaz):
     Toi[2,3]:= 0:
     Toi[2,4]:= 0:
     Toi[3,1]:= 0:
     Toi[3,2]:= 0:
     Toi[3,3]:= 1:
     Toi
   end_proc:



// Procedure giving the homogeneous transformation matrix of the rotation by an angle theta
//  about an arbitrary axis whose direction cosines are equal to nx, ny and nz.
Trotn:=proc(nx,ny,nz,theta)
   local Toi, M;
   begin
     M:=Dom::Matrix():
     Toi:=M(4,4,[[],[],[],[0,0,0,1]]):
     Toi[1,1]:= nx*nx*(1-cos(theta))+cos(theta):
     Toi[1,2]:= nx*ny*(1-cos(theta))-nz*sin(theta):
     Toi[1,3]:= nx*nz*(1-cos(theta))-ny*sin(theta):
     Toi[1,4]:= 0:
     Toi[2,1]:= nx*ny*(1-cos(theta))+nz*sin(theta):
     Toi[2,2]:= ny*ny*(1-cos(theta))+cos(theta):
     Toi[2,3]:= ny*nz*(1-cos(theta))-nx*sin(theta):
     Toi[2,4]:= 0:
     Toi[3,1]:= nx*nz*(1-cos(theta))-ny*sin(theta):
     Toi[3,2]:= nx*nz*(1-cos(theta))+nx*sin(theta):
     Toi[3,3]:= nz*nz*(1-cos(theta))+cos(theta):
     Toi
   end_proc:



// Procedure giving the homogeneous transformation matrix of the displacement about its coordinates x, y and z.
Tdisp:=proc(x,y,z)
   local Toi, M;
   begin
     M:=Dom::Matrix():
     Toi:=M(4,4,[[1,0,0,x],[0,1,0,y],[0,0,1,z],[0,0,0,1]]):
     Toi
   end_proc:
   


// Counter initialization.
time0:=time()/1000:



// Option flags initialization.
SIMPLIFY:=1:
ADVANCED:=0:
ANIM:=0:
FORCES:=0:
STATIC:=0:
PLOT:=0:
LATEX_FR:=0:
LATEX_EN:=0:



// Import of the LINALG library relative to the linear algebra.
export(linalg):



// Definition of maximum number of bodies and configuration parameters.
nbrbodymax:= 55:
nbrdofmax:= 25:
nbrdepmax:= 25:



// Definition of used types of user's file.
Matrice:=Dom::Matrix():
BodyRef:=array(0..nbrbodymax-1):
T0G:=array(0..nbrbodymax-1):
TrefG:=array(0..nbrbodymax-1):
mass:=array(0..nbrbodymax-1):
IG:=array(0..nbrbodymax-1):
gravity:=Matrice(3,1):
for i from 0 to nbrbodymax-1 do
T0G[i]:=Matrice(4,4):
TrefG[i]:=Matrice(4,4):
BodyRef[i]:=-1:
IG[i]:=Matrice(3,3,[[Ixx[i],Ixy[i],Ixz[i]],[Iyx[i],Iyy[i],Iyz[i]],[Izx[i],Izy[i],Izz[i]]]):
Ixx[i]:=0:
Ixy[i]:=0:
Ixz[i]:=0:
Iyy[i]:=0:
Iyz[i]:=0:
Izz[i]:=0:
end_for:
for i from 0 to nbrdofmax-1 do
qi[i]:=0:
qdi[i]:=0:
end_for:
nbrdep:= 0:



// Reading and writing path where the results must be save.
repertoire:=FichierUtil:
while substring(repertoire,length(repertoire)-1) <> "/" do
repertoire:=substring(repertoire,0..length(repertoire)-2):
end_while:
FichierUtil:=substring(FichierUtil,(length(repertoire)),(length(FichierUtil)-length(repertoire))):
filename:=FichierUtil:
while substring(filename,length(filename)-1) <> "." do
filename:=substring(filename,0..length(filename)-2):
end_while:
filename:=substring(filename,0..length(filename)-2):
READPATH:=repertoire:
WRITEPATH:=repertoire:



// Reading of user's file.
read(FichierUtil):



// Degrees of freedom maximum index (from 0 to nbrdof-1).
// Bodies maximum index (from 0 to nbrbody-1).
// Dependant variables maximum index (from 0 to nbrdep-1).
indexdof := nbrdof - 1:
indexbody := nbrbody - 1:
indexdep := nbrdep - 1:



// CPU time of data reading from user's file.
timeRD:=time()/1000:
ssRD:=trunc(timeRD-time0):
print(Unquoted,"... Reading user's file in ".ssRD." second(s).\n"):



// Actualized memory allocation.
for i from nbrbody to nbrbodymax-1 do
delete(T0G[i]):
delete(TrefG[i]):
delete(mass[i]):
delete(IG[i]):
end_for:
for i from nbrdof to nbrdofmax-1 do
delete(qi[i]):
delete(qdi[i]):
end_for:



// Testing existence of relative matrix
nbr_relative:=0:
zero_matrice:=Matrice(4,4):
index_relative:=array(0..indexbody):
for i from 0 to indexbody do
if T0G[i] = zero_matrice and BodyRef[i] <> -1 then
index_relative[i]:=1:
T0G[i]:=TrefG[i]:
nbr_relative:=nbr_relative+1:
else
index_relative[i]:=0:
end_if:
end_for:



// Following definitions of other types.
T0Gd:=array(0..indexbody):
R0G:=array(0..indexbody):
R0Gd:=array(0..indexbody):
MatRot:=array(0..indexbody):
omega:=array(0..indexbody):
omegad:=array(0..indexbody):
vG:=array(0..indexbody):
aG:=array(0..indexbody):
for i from 0 to indexbody do
T0Gd[i]:=Matrice(4,4,[[],[],[],[0,0,0,1]]):
R0G[i]:=Matrice(3,3):
R0Gd[i]:=Matrice(3,3):
MatRot[i]:=Matrice(3,3):
omega[i]:=Matrice(3,1):
omegad[i]:=Matrice(3,1):
vG[i]:=Matrice(3,1):
aG[i]:=Matrice(3,1):
R[i]:=Matrice(3,1):
MG[i]:=Matrice(3,1):
Iyx[i]:=Ixy[i]:
Izx[i]:=Ixz[i]:
Izy[i]:=Iyz[i]:
end_for:



// Declaration of dependant variables.
if nbrdep > 0 then
for k from 0 to indexdep do
pi[k]:=p[k]:
delete(p[k]):
T0G:=subs(T0G,p[k]=p[k](t)):
end_for:
end_if:



// Declaration of partial velocities.
if ADVANCED = 1 then
for i from 0 to indexbody do
vGpartial[i]:=Matrice(3,nbrdof):
omegapartial[i]:=Matrice(3,nbrdof):
end_for:
end_if:



// Creation of time dependence.
for i from 0 to indexdof do
T0G:=subs(T0G,q[i]=q[i](t)):
if nbrdep > 0 then
pi:=subs(pi,q[i]=q[i](t)):
end_if:
end_for:



// Calculed kinematics (part one).
for i from 0 to indexbody do
T0Gd[i]:=diff(T0G[i],t):
vG[i]:=T0Gd[i]*Matrice(4,1,[0,0,0,1]):
vG[i]:=vG[i][1..3,1..1]:
R0G[i]:=T0G[i][1..3,1..3]:
R0Gd[i]:=T0Gd[i][1..3,1..3]:
MatRot[i]:=R0Gd[i]*transpose(R0G[i]):
omega[i]:=Matrice([[MatRot[i][3,2]],[MatRot[i][1,3]],[MatRot[i][2,1]]]):
end_for:
for k from 0 to indexdep do
pdi[k]:=diff(pi[k],t):
end_for:
timeKI1:=time()/1000:
ssKI1:=trunc(timeKI1-timeRD):



// Simplification of kinematics parameters (part one).
if SIMPLIFY=1 then
for i from 0 to indexbody do
T0G[i]:=simplify(T0G[i]):
T0Gd[i]:=simplify(T0Gd[i]):
vG[i]:=simplify(vG[i]):
omega[i]:=simplify(omega[i]):
end_for:
for k from 0 to indexdep do
pdi[k]:=simplify(pdi[k]):
end_for:
end_if:
timeSP1:=time()/1000:
ssSP1:=trunc(timeSP1-timeKI1):



// Complete calculed kinematics (part two).
for i from 0 to indexbody do
aG[i]:=diff(vG[i],t):
omegad[i]:=diff(omega[i],t):
end_for:
for k from 0 to indexdep do
pddi[k]:=diff(pdi[k],t):
end_for:
timeKI2:=time()/1000:
ssKI2:=trunc(timeKI2-timeSP1):



// Simplification of kinematics parameters (part two).
if SIMPLIFY=1 then
for i from 0 to indexbody do
aG[i]:=simplify(aG[i]):
omegad[i]:=simplify(omegad[i]):
end_for:
for k from 0 to indexdep do
pddi[k]:=simplify(pddi[k]):
end_for:
end_if:
timeSP2:=time()/1000:
ssSP2:=trunc(timeSP2-timeKI2):



// Form modification of configuration parameters.
for i from 0 to indexbody do
for j from 0 to indexdof do
vG[i]:=subs(vG[i],diff(q[j](t),t)=qd[j],diff(q[j](t),t,t)=qdd[j],q[j](t)=q[j]):
omega[i]:=subs(omega[i],diff(q[j](t),t)=qd[j],diff(q[j](t),t,t)=qdd[j],q[j](t)=q[j]):
aG[i]:=subs(aG[i],diff(q[j](t),t)=qd[j],diff(q[j](t),t,t)=qdd[j],q[j](t)=q[j]):
omegad[i]:=subs(omegad[i],diff(q[j](t),t)=qd[j],diff(q[j](t),t,t)=qdd[j],q[j](t)=q[j]):
T0G[i]:=subs(T0G[i],diff(q[j](t),t)=qd[j],diff(q[j](t),t,t)=qdd[j],q[j](t)=q[j]):
T0Gd[i]:=subs(T0Gd[i],diff(q[j](t),t)=qd[j],diff(q[j](t),t,t)=qdd[j],q[j](t)=q[j]):
end_for:
end_for:
if nbrdep > 0 then
for i from 0 to indexbody do
for k from 0 to indexdep do
vG[i]:=subs(vG[i],diff(p[k](t),t)=pd[k],diff(p[k](t),t,t)=pdd[k],p[k](t)=p[k]):
omega[i]:=subs(omega[i],diff(p[k](t),t)=pd[k],diff(p[k](t),t,t)=pdd[k],p[k](t)=p[k]):
aG[i]:=subs(aG[i],diff(p[k](t),t)=pd[k],diff(p[k](t),t,t)=pdd[k],p[k](t)=p[k]):
omegad[i]:=subs(omegad[i],diff(p[k](t),t)=pd[k],diff(p[k](t),t,t)=pdd[k],p[k](t)=p[k]):
T0G[i]:=subs(T0G[i],diff(p[k](t),t)=pd[k],diff(p[k](t),t,t)=pdd[k],p[k](t)=p[k]):
T0Gd[i]:=subs(T0Gd[i],diff(p[k](t),t)=pd[k],diff(p[k](t),t,t)=pdd[k],p[k](t)=p[k]):
end_for:
end_for:
for k from 0 to indexdep do
for j from 0 to indexdof do
pi[k]:=subs(pi[k],q[j](t)=q[j]):
pdi[k]:=subs(pdi[k],diff(q[j](t),t)=qd[j],q[j](t)=q[j]):
pddi[k]:=subs(pddi[k],diff(q[j](t),t)=qd[j],diff(q[j](t),t,t)=qdd[j],q[j](t)=q[j]):
end_for:
end_for:
end_if:


// Complete calculed kinematics (part three).
if ADVANCED = 1 then
temp:=Matrice(3,1):
for i from 0 to indexbody do
for j from 0 to indexdof do
vGpartial[i,j]:=diff(vG[i],qd[j]):
omegapartial[i,j]:=diff(omega[i],qd[j]):
for k from 0 to indexdep do
temp:=diff(vG[i],pd[k])*diff(pdi[k],qd[j]):
vGpartial[i,j]:=vGpartial[i,j]+temp:
temp:=diff(omega[i],pd[k])*diff(pdi[k],qd[j]):
omegapartial[i,j]:=omegapartial[i,j]+temp:
end_for:
end_for:
end_for:
end_if:
timeKI3:=time()/1000:
SSKI3:=trunc(timeKI3-timeSP2):
ssKI:=ssKI1+ssKI2+SSKI3:
print(Unquoted,"... Kinematics computation finished ".ssKI." second(s).\n"):



// Simplification of kinematics parameters (part three).
if ADVANCED = 1 then
if SIMPLIFY = 1 then
for i from 0 to indexbody do
for j from 0 to indexdof do
aG[i]:=simplify(aG[i]):
omegapartial[i,j]:=simplify(omegapartial[i,j]):
end_for:
end_for:
end_if:
end_if:
timeSP3:=time()/1000:
ssSP3:=trunc(timeSP3-timeKI3):
ssSP:=ssSP1+ssSP2+ssSP3:
print(Unquoted,"... Kinematics simplification finished in ".ssSP." second(s).\n"):



// g??ation of the complete C++ code.
fileCPP:=filename.".cpp":
f:="// File in C++ format \n// ".title."\n \n \n":
f:=f."//    copyright (C) 2003 Olivier VERLINDEN\n":
f:=f."//    Service de Mecanique rationnelle, Dynamique et Vibrations\n":
f:=f."//    Faculte Polytechnique de Mons\n":
f:=f."//    31, Bd Dolez, 7000 MONS (Belgium)\n":
f:=f."//    Olivier.Verlinden@fpms.ac.be\n":
f:=f."//\n":
f:=f."// This file is part of EasyDyn\n":
f:=f."//\n":
f:=f."// EasyDyn is free software; you can redistribute it and/or modify it under the\n":
f:=f."// terms of the GNU GeneralPublic License as published by the Free Software\n":
f:=f."// Foundation; either version 2, or (at your option) any later version.\n":
f:=f."//\n":
f:=f."// EasyDyn is distributed in the hope that it will be useful, but WITHOUT ANY\n":
f:=f."// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR\n":
f:=f."// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.\n":
f:=f."//\n":
f:=f."// You should have received a copy of the GNU General Public License along with \n":
f:=f."// EasyDyn; see the file COPYING.  If not, write to the Free Software\n":
f:=f."// Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.\n":
f:=f."//\n":
f:=f."//\n\n\n":
f:=f."#define EASYDYNMBSMAIN  // to declare the global variable \n":
if ADVANCED = 1 then
f:=f."#define EASYDYNMBSADVANCED  // to declare the global variable \n":
end_if:
f:=f."#include <math.h> \n":
f:=f."#include <EasyDyn/mbs.h> \n":
if ANIM=1 then
f:=f."#include <EasyDyn/visu.h> \n":
f:=f."#include <EasyDyn/vec.h> \n":
f:=f."#include <fstream> \n\n":
f:=f."using namespace std ; \n":
f:=f."scene thescene ; \n":
f:=f."ofstream VanFile ; \n\n":
end_if:
f:=f."\n\n//---------------------------------------------------\n":
f:=f."void WriteDataHeader(ostream &OutFile) \n{\n":
f:=f."WriteStateVariablesHeader(OutFile) ;\n":
f:=f."OutFile << endl ;\n":
f:=f."}\n":
f:=f."\n//----------------------------------------------------\n":
f:=f."void SaveData(ostream &OutFile) \n{\n":
f:=f."SaveStateVariables(OutFile) ;\n":
f:=f."OutFile << endl ;\n":
if ANIM=1 then
f:=f."thescene.WriteCoord(VanFile) ; \n":
end_if:
f:=f."}\n":
f:=f."\n//----------------------------------------------------\n":
f:=f."void SetInertiaData() \n{\n":
co(1):="x":
co(2):="y":
co(3):="z":
for i from 0 to indexbody do
f:=f."// Inertia for body ".i."\n":
gen:=generate::C(hold(p)=mass[i]):
f:=f."body[".i."].mass".substring(gen,5,length(gen)-5)."\n":
gen:=generate::C(hold(p)=IG[i][1,1]):
f:=f."body[".i."].PhiG.Ixx".substring(gen,5,length(gen)-5)."\n":
gen:=generate::C(hold(p)=IG[i][2,2]):
f:=f."body[".i."].PhiG.Iyy".substring(gen,5,length(gen)-5)."\n":
gen:=generate::C(hold(p)=IG[i][3,3]):
f:=f."body[".i."].PhiG.Izz".substring(gen,5,length(gen)-5)."\n":
gen:=generate::C(hold(p)=IG[i][1,2]):
f:=f."body[".i."].PhiG.Ixy".substring(gen,5,length(gen)-5)."\n":
gen:=generate::C(hold(p)=IG[i][1,3]):
f:=f."body[".i."].PhiG.Ixz".substring(gen,5,length(gen)-5)."\n":
gen:=generate::C(hold(p)=IG[i][2,3]):
f:=f."body[".i."].PhiG.Iyz".substring(gen,5,length(gen)-5)."\n":
end_for:
f:=f."}\n":
f:=f."\n//----------------------------------------------------\n":
f:=f."void ComputeMotion() \n{\n":
if nbrdep > 0 then
f:=f."// Dependant variables\n":
for k from 0 to indexdep do
gen:=generate::C(hold(p)=pi[k]):
f:=f."p[".k."]".substring(gen,5,length(gen)-5)."\n":
gen:=generate::C(hold(p)=pdi[k]):
f:=f."pd[".k."]".substring(gen,5,length(gen)-5)."\n":
gen:=generate::C(hold(p)=pddi[k]):
f:=f."pdd[".k."]".substring(gen,5,length(gen)-5)."\n":
end_for:
end_if:
f:=f."// Homogenous transformation matrices of each body\n":
for i from 0 to indexbody do
if index_relative[i]=0 then
for j from 1 to 3 do
for k from 1 to 3 do
gen:=generate::C(hold(p)=T0G[i][j,k]):
f:=f."body[".i."].T0G.R.r".j.k.substring(gen,5,length(gen)-5)."\n":
end_for:
end_for:
k:=4:
for j from 1 to 3 do
gen:=generate::C(hold(p)=T0G[i][j,k]):
f:=f."body[".i."].T0G.e.".co(j).substring(gen,5,length(gen)-5)."\n":
end_for:
else
for j from 1 to 3 do
for k from 1 to 3 do
gen:=generate::C(hold(p)=T0G[i][j,k]):
f:=f."body[".i."].TrefG.R.r".j.k.substring(gen,5,length(gen)-5)."\n":
end_for:
end_for:
k:=4:
for j from 1 to 3 do
gen:=generate::C(hold(p)=T0G[i][j,k]):
f:=f."body[".i."].TrefG.e.".co(j).substring(gen,5,length(gen)-5)."\n":
end_for:
end_if:
end_for:
f:=f."// Velocity vectors of the center of gravity of each body\n":
for i from 0 to indexbody do
if index_relative[i]=0 then
for j from 1 to 3 do
gen:=generate::C(hold(p)=vG[i][j]):
f:=f."body[".i."].vG.".co(j).substring(gen,5,length(gen)-5)."\n":
end_for:
else
for j from 1 to 3 do
gen:=generate::C(hold(p)=vG[i][j]):
f:=f."body[".i."].vGrel.".co(j).substring(gen,5,length(gen)-5)."\n":
end_for:
end_if:
end_for:
f:=f."// Rotational velocity vectors of each body\n":
for i from 0 to indexbody do
if index_relative[i]=0 then
for j from 1 to 3 do
gen:=generate::C(hold(p)=omega[i][j]):
f:=f."body[".i."].omega.".co(j).substring(gen,5,length(gen)-5)."\n":
end_for:
else
for j from 1 to 3 do
gen:=generate::C(hold(p)=omega[i][j]):
f:=f."body[".i."].omegarel.".co(j).substring(gen,5,length(gen)-5)."\n":
end_for:
end_if:
end_for:
f:=f."// Acceleration vectors of the center of gravity of each body\n":
for i from 0 to indexbody do
if index_relative[i]=0 then
for j from 1 to 3 do
gen:=generate::C(hold(p)=aG[i][j]):
f:=f."body[".i."].aG.".co(j).substring(gen,5,length(gen)-5)."\n":
end_for:
else
for j from 1 to 3 do
gen:=generate::C(hold(p)=aG[i][j]):
f:=f."body[".i."].aGrel.".co(j).substring(gen,5,length(gen)-5)."\n":
end_for:
end_if:
end_for:
f:=f."// Rotational acceleration vectors of each body\n":
for i from 0 to indexbody do
if index_relative[i]=0 then
for j from 1 to 3 do
gen:=generate::C(hold(p)=omegad[i][j]):
f:=f."body[".i."].omegad.".co(j).substring(gen,5,length(gen)-5)."\n":
end_for:
else
for j from 1 to 3 do
gen:=generate::C(hold(p)=omegad[i][j]):
f:=f."body[".i."].omegadrel.".co(j).substring(gen,5,length(gen)-5)."\n":
end_for:
end_if:
end_for:
if index_relative[0]=1 then
f:=f."// Definition of relative motion\n":
end_if:
for i from 0 to indexbody do
if index_relative[i]=1 then
gen:=generate::C(hold(p)=BodyRef[i]):
f:=f."ComposeMotion(".i.",".BodyRef[i].");\n":
end_if:
end_for:
f:=f."}\n":
if ADVANCED = 1 then
f:=f."\n//----------------------------------------------------\n":
f:=f."void ComputePartialVelocities() \n{\n":
for i from 0 to indexbody do
f:=f."// Kinematics for body ".i."\n":
temp[1]:=0:
temp[2]:=0:
temp[3]:=0:
for j from 0 to indexdof do
if vGpartial[i,j] <> temp then
for k from 1 to 3 do
gen:=generate::C(hold(p)=vGpartial[i,j][k]):
if index_relative[i]=0 then
f:=f."body[".i."].vGpartial[".j."].".co(k).substring(gen,5,length(gen)-5)."\n":
else
f:=f."body[".i."].vGrelpartial[".j."].".co(k).substring(gen,5,length(gen)-5)."\n":
end_if:
end_for:
end_if:
if omegapartial[i,j] <> temp then
for k from 1 to 3 do
gen:=generate::C(hold(p)=omegapartial[i,j][k]):
if index_relative[i]=0 then
f:=f."body[".i."].omegapartial[".j."].".co(k).substring(gen,5,length(gen)-5)."\n":
else
f:=f."body[".i."].omegarelpartial[".j."].".co(k).substring(gen,5,length(gen)-5)."\n":
end_if:
end_for:
end_if:
end_for:
if index_relative[i]=1 then
gen:=generate::C(hold(p)=BodyRef[i]):
f:=f."ComposePartialVelocities(".i.",".BodyRef[i].");\n":
end_if:
end_for:
f:=f."}\n":
end_if:
f:=f."\n//----------------------------------------------------\n":
f:=f."void AddAppliedEfforts() \n{\n":
f:=f."// Contribution of gravity\n":
gen:=generate::C(hold(p)=gravity[1]):
f:=f."vec gravity(".substring(gen,8,length(gen)-10).",":
gen:=generate::C(hold(p)=gravity[2]):
f:=f.substring(gen,8,length(gen)-10).",":
gen:=generate::C(hold(p)=gravity[3]):
f:=f.substring(gen,8,length(gen)-10).") ;\n":
f:=f."AddGravityForces(gravity) ;\n":
if FORCES=1 then
f:=f."// Contribution of external forces\n":
f:=f."#include \"".filename.".AppEff.cpp\" \n":
end_if:
f:=f."}\n\n//-----------------------------------------------------\n":
f:=f."void ComputeResidual()\n":
f:=f."{\nComputeResidualmbs();\n}\n":
f:=f."\n\n//-----------------------------------------------------\n":
f:=f."\n\nint main()\n{\n":
f:=f."// Initialisation and memory allocation\n":
gen:=generate::C(hold(nbrdof)=nbrdof):
f:=f.substring(gen,4,length(gen)-8)." ;\n":
gen:=generate::C(hold(nbrbody)=nbrbody):
f:=f.substring(gen,4,length(gen)-8)." ;\n":
if nbrdep > 0 then
gen:=generate::C(hold(nbrdep)=nbrdep):
f:=f.substring(gen,4,length(gen)-8)." ;\n":
end_if:
f:=f."application = \"".substring(FichierUtil,0,length(FichierUtil)-3)."\" ;\n":
f:=f."InitEasyDynmbs() ;\n":
if ANIM=1 then
f:=f."// Let's create the shapes \n":
f:=f."shape *s0 ":
for i from 1 to indexbody do
f:=f.", *s".i." ":
end_for:
f:=f."; \n":
for i from 0 to indexbody do
f:=f."s".i." = new box(&(body[".i."].T0G),vcoord(-0.1,-0.1,-0.1),vcoord(0.1,0.1,0.1),1,2) ; \n":
end_for:
f:=f."// Let's add the shapes to the scene \n":
for i from 0 to indexbody do
f:=f."thescene.AddShape(s".i.") ;\n":
end_for:
f:=f."// Uncomment the following line if you want a moving observer \n":
f:=f."// thescene.SetVisuFrame(&Tref) ; \n":
f:=f."// Let's open an animation file \n":
f:=f."VanFile.open(\"".filename.".van\") ; \n":
end_if:
f:=f."// Initial configuration\n":
for j from 0 to indexdof do
qi[j]:= float(qi[j]):
gen:=generate::C(hold(p)=qi[j]):
f:=f."q[".j."]".substring(gen,5,length(gen)-5)."\n":
gen:=generate::C(hold(p)=qdi[j]):
f:=f."qd[".j."]".substring(gen,5,length(gen)-5)."\n":
end_for:
if STATIC=1 then
f:=f."// Let's first search for the static equilibrium \n":
f:=f."int *doflocked ; \n":
f:=f."doflocked=new int[nbrdof] ; \n":
for j from 0 to indexdof do
f:=f."doflocked[".j."]=0 ; \n":
end_for:
f:=f."StaticEquilibrium(doflocked) ; \n":
end_if:
if ANIM=1 then
f:=f."// Let's save the structure of the scene \n":
f:=f."ComputeMotion() ; \n":
f:=f."thescene.CreateVolFile(\"".filename.".vol\") ; \n":
end_if:
f:=f."// Let's go !\n":
gen:=generate::C(hold(p)=FinalTime):
f:=f."NewmarkIntegration(".substring(gen,8,length(gen)-10):
gen:=generate::C(hold(p)=StepSave):
f:=f.",".substring(gen,8,length(gen)-10):
gen:=generate::C(hold(p)=StepMax):
f:=f.",".substring(gen,8,length(gen)-10).") ;\n":
f:=f."// The clean way to finish \n":
f:=f."EndEasyDynmbs() ;\n":
if ANIM=1 then
f:=f."VanFile.close() ; \n":
end_if:
f:=f."}\n":
f:=f."\n//-----------------------------------------------------\n \n":
fprint(Unquoted,Text,fileCPP,f):



// CPU-time of the complete C++ file writing.
timeCPP:=time()/1000:
if SIMPLIFY=1 then
ssCPP:=trunc(timeCPP-timeSP3):
else
ssCPP:=trunc(timeCPP-timeKI3):
ssSP:=0:
end_if:
print(Unquoted,"... Writing in C++ file finished in ".ssCPP." second(s).\n"):



// generation of the Gnuplot code.
if PLOT=1 then
filePLT:=filename.".plt":
f:="reset \n":
f:=f."set xlabel \"Time [s]\"  \n":
f:=f."set grid \n\n":
if LATEX_FR=1 or LATEX_EN=1 then
f:=f."set term postscript eps color \"Times-Roman\" 20 \n":
f:=f."set output \"figure1.eps\"  \n":
end_if:
f:=f."set ylabel \"displacements\"  \n":
f:=f."plot '".filename.".res' using 1:2 title 'q_0' with line ":
for j from 1 to indexdof do
k:=2+3*j:
f:=f.", '".filename.".res' using 1:".k." title 'q_".j."' with line ":
end_for:
if sysname(Arch)="MSDOS" then
f:=f."\nset term win \n":
end_if:
if sysname(Arch)="linux" then
f:=f."\nset term X11 \n":
end_if:
f:=f."replot \n":
f:=f."pause -1 'Next plot (velocity level)?' \n\n":
if LATEX_FR=1 or LATEX_EN=1 then
f:=f."set term postscript eps color \"Times-Roman\" 20 \n":
f:=f."set output \"figure2.eps\"  \n":
end_if:
f:=f."set ylabel \"velocities\"  \n":
f:=f."plot '".filename.".res' using 1:3 title 'qd_0' with line ":
for j from 1 to indexdof do
k:=3+3*j:
f:=f.", '".filename.".res' using 1:".k." title 'qd_".j."' with line ":
end_for:
if sysname(Arch)="MSDOS" then
f:=f."\nset term win \n":
end_if:
if sysname(Arch)="linux" then
f:=f."\nset term X11 \n":
end_if:
f:=f."replot \n":
f:=f."pause -1 'Next plot (acceleration level)?' \n\n":
if LATEX_FR=1 or LATEX_EN=1 then
f:=f."set term postscript eps color \"Times-Roman\" 20 \n":
f:=f."set output \"figure3.eps\"  \n":
end_if:
f:=f."set ylabel \"accelerations\"  \n":
f:=f."plot '".filename.".res' using 1:4 title 'qdd_0' with line ":
for j from 1 to indexdof do
k:=4+3*j:
f:=f.", '".filename.".res' using 1:".k." title 'qdd_".j."' with line ":
end_for:
if sysname(Arch)="MSDOS" then
f:=f."\nset term win \n":
end_if:
if sysname(Arch)="linux" then
f:=f."\nset term X11 \n":
end_if:
f:=f."replot \n":
fprint(Unquoted,Text,filePLT,f):
timePLT:=time()/1000:
ssPLT:=trunc(timePLT-timeCPP):
print(Unquoted,"... Writing in GNUPLOT file finished in ".ssPLT." second(s).\n"):
end_if:



// Generation of the TEX code in French.
if LATEX_FR=1 then
fileTEXFR:="rapport_".filename.".tex":
f:="\\documentclass[a4paper,11pt]{article}\n":
f:=f."\\topmargin -0.5cm\n":
f:=f."\\headheight 0.7cm\n":
f:=f."\\headsep    0.8cm\n":
f:=f."\\topskip      0cm\n":
f:=f."\\textheight  24cm\n":
f:=f."\\oddsidemargin  0cm\n":
f:=f."\\evensidemargin 0cm\n":
f:=f."\\textwidth   16cm\n":
f:=f."\\usepackage{verbatim}\n":
f:=f."\\usepackage{array} %>{$}m{3cm}<{$}\n":
f:=f."\\usepackage{amssymb}\n":
f:=f."\\usepackage{amsmath}\n":
f:=f."\\usepackage[dvips]{graphicx}\n":
f:=f."\\usepackage[isolatin]{inputenc}\n":
f:=f."\\usepackage[francais]{babel}\n":
f:=f."\\usepackage{fancyhdr}\n":
f:=f."\\pagestyle{fancy}\n":
f:=f."\\fancyhead{}\n":
f:=f."\\fancyfoot{}\n":
f:=f."\\fancyhead[L]{".title."}\n":
f:=f."\\fancyhead[R]{\\thepage}\n":
f:=f."\\title{\\textsf{".title."}}\n":
f:=f."\\author{}\n":
f:=f."\\begin{document}\n":
f:=f."\\sloppy\n":
f:=f."\\maketitle\n\n":
f:=f."\\section{Donnes gnrales du mcanisme tudi}\n\n":
f:=f."Le nombre de solides du systme s'lve  ".nbrbody." (dfini par la variable globale \\texttt{nbrbody}). ":
f:=f."Par la suite, ils se dfinissent par $S_j$ avec $j$ allant de 0  ".indexbody.". ":
if nbrdof > 1 then
f:=f."Le nombre de degrs de libert s'lve, quant  lui,  ".nbrdof." (\\texttt{nbrdof}). ":
f:=f."Les paramtres de configuration sont donc dfinis par $q_i$ ($i$ allant de 0  ".indexdof.").\\\\\n\n":
else
f:=f."Le mcanisme analys est desmodrome ( un degr de libert) et donc ":
f:=f."un seul paramtre de configuration est dfini par $q_0$.\\\\\n\n":
end_if:
f:=f."Les donnes inertielles, dfinies dans le fichier utilisateur, se rsument aux masses $m_{Si}$ et ":
f:=f."aux tenseurs d'inertie $\\Phi_{G,Si}$ dfinis au centre de gravit de chaque solide concern.\\\\\n\n":
for i from 0 to indexbody do
gen:=generate::TeX(mass[i]):
f:=f."\\[ m_{S".i."}=".gen."\\,kg \\]\n":
end_for:
f:=f."\\\\\n":
for i from 0 to indexbody do
phiG[i]:=Matrice(3,3,[[Ixx[i],-Ixy[i],-Ixz[i]],[-Iyx[i],Iyy[i],-Iyz[i]],[-Izx[i],-Izy[i],Izz[i]]]):
gen:=generate::TeX(phiG[i]):
f:=f."\\[\\Phi_{G,S".i."}=".gen." \\quad \\textrm{, en $kg.m^2$}\\]\\\\\n":
end_for:
f:=f."\n\\section{Paramtres cinmatiques calculs sous MuPAD}\n\n":
gen:=ssKI+ssSP:
f:=f."Ces paramtres ont t calculs  partir du fichier utilisateur \\texttt{".FichierUtil."} et ce, \n":
f:=f."avec un temps de calcul \\emph{CPU} de ".gen." seconde":
if gen>1 then f:=f."s" end_if:
f:=f.". \\\\\n\n":
if nbr_relative <> 0 then
if nbr_relative = 1 then
for i from 0 to indexbody do
if index_relative[i]=1 then
f:=f."\\noindent \\emph{Remarque} : Le mouvement du solide $S_{".i."}$ est un mouvement relatif et est dfini ":
f:=f."par rapport  celui de $S_{".BodyRef[i]."}$. \\\\\n\n":
end_if:
end_for:
else
f:=f."\\subsection*{Les mouvements relatifs}\n\n":
f:=f."Certains mouvements de solides ont t dfinis de manire relative par rapport  d'autres solides. ":
f:=f."Ces solides sont au nombre de ".nbr_relative." et se dfinissent de la manire suivante :\n":
f:=f."\\begin{itemize}\n":
for i from 0 to indexbody do
if index_relative[i]=1 then
f:=f."   \\item Le mouvement du solide $S_{".i."}$ est dfini par rapport  celui de $S_{".BodyRef[i]."}$.\n":
end_if:
end_for:
f:=f."\\end{itemize} \n\n":
end_if:
end_if:
if nbrdep > 0 then
f:=f."\\subsection*{Les variables dpendantes}\n\n":
f:=f."L'utilisation de variables intermdiaires s'est avre ncessaire afin de rduire les temps de calculs. ":
f:=f."Elles sont au nombre de ".nbrdep." (dfini par $p_i$, $i$ allant de 0  ".indexdep.") :":
for k from 0 to indexdep do
gen:=generate::TeX(pi[k]):
f:=f."\\[p_".k."=".gen."\\]\\\\\n\n":
gen:=generate::TeX(pdi[k]):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\dot{p}_".k."=".gen."\\]\\\\\n\n":
gen:=generate::TeX(pddi[k]):
gen:=stringlib::subs(gen,"qdd"="$\\ddot{q}$"):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\ddot{p}_".k."=".gen."\\]\\\\\n\n":
end_for:
end_if:
f:=f."\\subsection*{Les matrices de transformation homogne pour chaque solide}\n\n":
for i from 0 to indexbody do
if index_relative[i]=0 then
gen:=generate::TeX(T0G[i]):
f:=f."\\[T_{0G,S".i."}=".gen."\\]\\\\\n\n":
else
gen:=generate::TeX(T0G[i]):
f:=f."\\[T_{refG,S".i."/S".BodyRef[i]."}=".gen."\\]\\\\\n\n":
end_if:
end_for:
f:=f."\\subsection*{Leur drive par rapport au temps}\n\n":
for i from 0 to indexbody do
if index_relative[i]=0 then
gen:=generate::TeX(T0Gd[i]):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\dot{T}_{0G,S".i."}=".gen."\\]\\\\\n\n":
else
gen:=generate::TeX(T0Gd[i]):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\dot{T}_{refG,S".i."/S".BodyRef[i]."}=".gen."\\]\\\\\n\n":
end_if:
end_for:
f:=f."\\subsection*{Les vitesses des centres de gravit des solides}\n\n":
for i from 0 to indexbody do
if index_relative[i]=0 then
gen:=generate::TeX(vG[i]):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\vec{v}_{G,S".i."}=".gen."\\]\\\\\n\n":
else
gen:=generate::TeX(vG[i]):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\left\\{\\vec{v}_{G,S".i."/S".BodyRef[i]."}\\right\\}_{S".BodyRef[i]."}=".gen."\\]\\\\\n\n":
end_if:
end_for:
f:=f."\\subsection*{Les acclrations des centres de gravit des solides}\n\n":
for i from 0 to indexbody do
if index_relative[i]=0 then
gen:=generate::TeX(aG[i]):
gen:=stringlib::subs(gen,"qdd"="$\\ddot{q}$"):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\vec{a}_{G,S".i."}=".gen."\\]\\\\\n\n":
else
gen:=generate::TeX(aG[i]):
gen:=stringlib::subs(gen,"qdd"="$\\ddot{q}$"):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\left\\{\\vec{a}_{G,S".i."/S".BodyRef[i]."}\\right\\}_{S".BodyRef[i]."}=".gen."\\]\\\\\n\n":
end_if:
end_for:
f:=f."\\subsection*{Les vitesses de rotation des solides}\n\n":
for i from 0 to indexbody do
if index_relative[i]=0 then
gen:=generate::TeX(omega[i]):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\vec{\\omega}_{S".i."}=".gen."\\]\\\\\n\n":
else
gen:=generate::TeX(omega[i]):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\left\\{\\vec{\\omega}_{S".i."/S".BodyRef[i]."}\\right\\}_{S".BodyRef[i]."}=".gen."\\]\\\\\n\n":
end_if:
end_for:
f:=f."\\subsection*{Les acclrations de rotation des solides}\n\n":
for i from 0 to indexbody do
if index_relative[i]=0 then
gen:=generate::TeX(omegad[i]):
gen:=stringlib::subs(gen,"qdd"="$\\ddot{q}$"):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\vec{\\dot{\\omega}}_{S".i."}=".gen."\\]\\\\\n\n":
else
gen:=generate::TeX(omegad[i]):
gen:=stringlib::subs(gen,"qdd"="$\\ddot{q}$"):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\left\\{\\vec{\\dot{\\omega}}_{S".i."/S".BodyRef[i]."}\\right\\}_{S".BodyRef[i]."}=".gen."\\]\\\\\n\n":
end_if:
end_for:
if nbrdep > 0 then
f:=stringlib::subs(f,"pdd"="$\\ddot{p}$"):
f:=stringlib::subs(f,"pd"="$\\dot{p}$"):
end_if:
if FORCES=1 then
f:=f."\n\\section{Dfinition des efforts externes}\n\n":
f:=f."Les efforts externes diffrent de la gravit ont t dfinis dans le fichiers \\texttt{".filename.".AppEff.cpp}":
f:=f.", inclus automatiquement dans le fichier \\texttt{".filename.".cpp} (procdure \\texttt{AddAppliedEfforts()}), ":
f:=f."et contenant le code suivant : \\\\\n\n":
f:=f."\\footnotesize\n":
f:=f."\\verbatiminput{".filename.".AppEff.cpp}\n":
f:=f."\\normalsize\n\n":
end_if:
f:=f."\n\\section{Simulation du mcanisme}\n\n":
f:=f."La simulation temporelle s'est faite via la procdure \\texttt{NewmarkIntegration}":
f:=f."(\\emph{FinalTime} , \\emph{StepSave} , \\emph{StepMax}) du fichier \\texttt{".fileCPP."} o :\n":
f:=f."\\begin{itemize}\n":
gen:=generate::TeX(FinalTime):
f:=f."   \\item \\emph{FinalTime} est la dure de la simulation (=".gen." $s$),\n":
gen:=generate::TeX(StepSave):
f:=f."   \\item \\emph{StepSave} est le pas de temps dans l'intgration numrique (=".gen." $s$),\n":
gen:=generate::TeX(StepMax):
f:=f."   \\item \\emph{StepMax} est un pas de temps limite (=".gen." $s$),\n":
f:=f."\\end{itemize}\n":
f:=f."\\noindent les conditions initiales tant ":
for k from 0 to indexdof do
if qi[k]<>0 then
gen:=generate::TeX(qi[k]):
f:=f."$q_{".k."} = ".gen."$, ":
end_if:
if qdi[k]<>0 then
gen:=generate::TeX(qdi[k]):
f:=f."$\\dot{q}_{".k."} = ".gen."$, ":
end_if:
end_for:
f:=f."les autres tant nulles.\\\\\n":
f:=f."\n\\section{Rsultats}\n\n":
f:=f."L'affichage des volutions temporelles des diffrents paramtres de configuration ainsi que leurs ":
f:=f."drives premire et seconde s'tablit assez facilement sous \\textsf{Gnuplot} comme nous le montrent ":
f:=f."les figures~\\ref{figure1} ~\\ref{figure3} cres par le code suivant :\\\\\n\n":
f:=f."\\footnotesize\n":
f:=f."\\verbatiminput{".filename.".plt}\n":
f:=f."\\normalsize\n\n":
f:=f."\\begin{figure}[h!tb]\n":
f:=f."   \\begin{center}\n":
f:=f."      \\includegraphics[width=0.75\\textwidth]{figure1.eps}\n":
f:=f."   \\end{center}\n":
f:=f."   \\caption{Evolution temporelle des paramtres de configuration}\n":
f:=f."   \\label{figure1}\n":
f:=f."\\end{figure}\n\n":
f:=f."\\begin{figure}[h!tb]\n":
f:=f."   \\begin{center}\n":
f:=f."      \\includegraphics[width=0.75\\textwidth]{figure2.eps}\n":
f:=f."   \\end{center}\n":
f:=f."   \\caption{Evolution temporelle des drives premires des paramtres de configuration}\n":
f:=f."   \\label{figure2}\n":
f:=f."\\end{figure}\n\n":
f:=f."\\begin{figure}[h!tb]\n":
f:=f."   \\begin{center}\n":
f:=f."      \\includegraphics[width=0.75\\textwidth]{figure3.eps}\n":
f:=f."   \\end{center}\n":
f:=f."   \\caption{Evolution temporelle des drives secondes des paramtres de configuration}\n":
f:=f."   \\label{figure3}\n":
f:=f."\\end{figure}\n\n":
f:=f."\n\\appendix\n\n":
f:=f."\\clearpage\n":
f:=f."\\section{Listing du fichier MuPAD}\n":
f:=f."\\footnotesize\n":
f:=f."\\verbatiminput{".FichierUtil."}\n\n":
f:=f."\n\\end{document}\n":
fprint(Unquoted,Text,fileTEXFR,f):
timeTEXFR:=time()/1000:
ssTEXFR:=trunc(timeTEXFR-timePLT):
print(Unquoted,"... Ecriture dans le fichier TeX termine en ".ssTEXFR." seconde(s).\n"):
end_if:




// Generation of the TEX code in English.
if LATEX_EN=1 then
fileTEXEN:="report_".filename.".tex":
f:="\\documentclass[a4paper,11pt]{article}\n":
f:=f."\\topmargin -0.5cm\n":
f:=f."\\headheight 0.7cm\n":
f:=f."\\headsep    0.8cm\n":
f:=f."\\topskip      0cm\n":
f:=f."\\textheight  24cm\n":
f:=f."\\oddsidemargin  0cm\n":
f:=f."\\evensidemargin 0cm\n":
f:=f."\\textwidth   16cm\n":
f:=f."\\usepackage{verbatim}\n":
f:=f."\\usepackage{array} %>{$}m{3cm}<{$}\n":
f:=f."\\usepackage{amssymb}\n":
f:=f."\\usepackage{amsmath}\n":
f:=f."\\usepackage[dvips]{graphicx}\n":
f:=f."\\usepackage[isolatin]{inputenc}\n":
f:=f."\\usepackage[english]{babel}\n":
f:=f."\\usepackage{fancyhdr}\n":
f:=f."\\pagestyle{fancy}\n":
f:=f."\\fancyhead{}\n":
f:=f."\\fancyfoot{}\n":
f:=f."\\fancyhead[L]{".title."}\n":
f:=f."\\fancyhead[R]{\\thepage}\n":
f:=f."\\title{\\textsf{".title."}}\n":
f:=f."\\author{}\n":
f:=f."\\begin{document}\n":
f:=f."\\sloppy\n":
f:=f."\\maketitle\n\n":
f:=f."\\section{General data of the studied mechanism}\n\n":
f:=f."The system comprises ".nbrbody." bodies (defined by the global variable \\texttt{nbrbody}). ":
f:=f."Each body is called $S_j$ ($j$ from 0 to ".indexbody."). ":
if nbrdof > 1 then
f:=f."The number of degrees of freedom of the system is ".nbrdof." (\\texttt{nbrdof}). ":
f:=f."The configuration parameters are denoted by $q_i$ ($i$ from 0 to ".indexdof.").\\\\\n\n":
else
f:=f."The studied mechanism is describe by one degree of freedom and so ":
f:=f."only one configuration parameter is defined by $q_0$.\\\\\n\n":
end_if:
f:=f."The inertial data, given by the user, consist of the mass $m_{Si}$ and the inertia tensor $\\Phi_{G,Si}$ ":
f:=f."of each body $i$ expressed with respect to the center of gravity.\\\\\n\n":
for i from 0 to indexbody do
gen:=generate::TeX(mass[i]):
f:=f."\\[ m_{S".i."}=".gen."\\,kg \\]\n":
end_for:
f:=f."\\\\\n":
for i from 0 to indexbody do
phiG[i]:=Matrice(3,3,[[Ixx[i],-Ixy[i],-Ixz[i]],[-Iyx[i],Iyy[i],-Iyz[i]],[-Izx[i],-Izy[i],Izz[i]]]):
gen:=generate::TeX(phiG[i]):
f:=f."\\[\\Phi_{G,S".i."}=".gen." \\quad \\textrm{, in $kg.m^2$}\\]\\\\\n":
end_for:
f:=f."\n\\section{Complete kinematics calculed by CAGeM}\n\n":
gen:=ssKI+ssSP:
f:=f."The following parameters have been calculated from the user's file \\texttt{".FichierUtil."} and with ":
f:=f."a \\emph{CPU} time of ".gen." second":
if gen>1 then f:=f."s" end_if:
f:=f.". \\\\\n\n":
if nbr_relative <> 0 then
if nbr_relative = 1 then
for i from 0 to indexbody do
if index_relative[i]=1 then
f:=f."\\noindent \\emph{Note}: The motion of body $S_{".i."}$ has been defined as a relative motion with respect to":
f:=f." body $S_{".BodyRef[i]."}$. \\\\\n\n":
end_if:
end_for:
else
f:=f."\\subsection*{Relative motions}\n\n":
f:=f."The motion of some bodies has been defined as a relative motion with respect to another body. It is ":
f:=f."the case of ".nbr_relative." bodies, where motion is defined in the following manner:\n":
f:=f."\\begin{itemize}\n":
for i from 0 to indexbody do
if index_relative[i]=1 then
f:=f."   \\item Motion of body $S_{".i."}$ is given with respect to body $S_{".BodyRef[i]."}$.\n":
end_if:
end_for:
f:=f."\\end{itemize} \n\n":
end_if:
end_if:
if nbrdep > 0 then
f:=f."\\subsection*{Dependent variables}\n\n":
f:=f."Dependent variables have been used to perform an efficient simulation. ":
f:=f.nbrdep." parameters have been defined, denoted by $p_i$ ($i$ from 0 to ".indexdep."):":
for k from 0 to indexdep do
gen:=generate::TeX(pi[k]):
f:=f."\\[p_".k."=".gen."\\]\\\\\n\n":
gen:=generate::TeX(pdi[k]):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\dot{p}_".k."=".gen."\\]\\\\\n\n":
gen:=generate::TeX(pddi[k]):
gen:=stringlib::subs(gen,"qdd"="$\\ddot{q}$"):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\ddot{p}_".k."=".gen."\\]\\\\\n\n":
end_for:
end_if:
f:=f."\\subsection*{Homogeneous transformation matrix of each body}\n\n":
for i from 0 to indexbody do
if index_relative[i]=0 then
gen:=generate::TeX(T0G[i]):
f:=f."\\[T_{0G,S".i."}=".gen."\\]\\\\\n\n":
else
gen:=generate::TeX(T0G[i]):
f:=f."\\[T_{refG,S".i."/S".BodyRef[i]."}=".gen."\\]\\\\\n\n":
end_if:
end_for:
f:=f."\\subsection*{The time derivative of these matrices}\n\n":
for i from 0 to indexbody do
if index_relative[i]=0 then
gen:=generate::TeX(T0Gd[i]):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\dot{T}_{0G,S".i."}=".gen."\\]\\\\\n\n":
else
gen:=generate::TeX(T0Gd[i]):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\dot{T}_{refG,S".i."/S".BodyRef[i]."}=".gen."\\]\\\\\n\n":
end_if:
end_for:
f:=f."\\subsection*{The velocity of the center of gravity of each body}\n\n":
for i from 0 to indexbody do
if index_relative[i]=0 then
gen:=generate::TeX(vG[i]):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\vec{v}_{G,S".i."}=".gen."\\]\\\\\n\n":
else
gen:=generate::TeX(vG[i]):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\left\\{\\vec{v}_{G,S".i."/S".BodyRef[i]."}\\right\\}_{S".BodyRef[i]."}=".gen."\\]\\\\\n\n":
end_if:
end_for:
f:=f."\\subsection*{The acceleration of the center of gravity of each body}\n\n":
for i from 0 to indexbody do
if index_relative[i]=0 then
gen:=generate::TeX(aG[i]):
gen:=stringlib::subs(gen,"qdd"="$\\ddot{q}$"):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\vec{a}_{G,S".i."}=".gen."\\]\\\\\n\n":
else
gen:=generate::TeX(aG[i]):
gen:=stringlib::subs(gen,"qdd"="$\\ddot{q}$"):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\left\\{\\vec{a}_{G,S".i."/S".BodyRef[i]."}\\right\\}_{S".BodyRef[i]."}=".gen."\\]\\\\\n\n":
end_if:
end_for:
f:=f."\\subsection*{The rotation velocity of each body}\n\n":
for i from 0 to indexbody do
if index_relative[i]=0 then
gen:=generate::TeX(omega[i]):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\vec{\\omega}_{S".i."}=".gen."\\]\\\\\n\n":
else
gen:=generate::TeX(omega[i]):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\left\\{\\vec{\\omega}_{S".i."/S".BodyRef[i]."}\\right\\}_{S".BodyRef[i]."}=".gen."\\]\\\\\n\n":
end_if:
end_for:
f:=f."\\subsection*{The rotation acceleration of each body}\n\n":
for i from 0 to indexbody do
if index_relative[i]=0 then
gen:=generate::TeX(omegad[i]):
gen:=stringlib::subs(gen,"qdd"="$\\ddot{q}$"):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\vec{\\dot{\\omega}}_{S".i."}=".gen."\\]\\\\\n\n":
else
gen:=generate::TeX(omegad[i]):
gen:=stringlib::subs(gen,"qdd"="$\\ddot{q}$"):
gen:=stringlib::subs(gen,"qd"="$\\dot{q}$"):
f:=f."\\[\\left\\{\\vec{\\dot{\\omega}}_{S".i."/S".BodyRef[i]."}\\right\\}_{S".BodyRef[i]."}=".gen."\\]\\\\\n\n":
end_if:
end_for:
if nbrdep > 0 then
f:=stringlib::subs(f,"pdd"="$\\ddot{p}$"):
f:=stringlib::subs(f,"pd"="$\\dot{p}$"):
end_if:
if FORCES=1 then
f:=f."\n\\section{Definition of external efforts}\n\n":
f:=f."External efforts different from gravity have been defined in the file \\texttt{".filename.".AppEff.cpp}":
f:=f.", automatically included in \\texttt{".filename.".cpp} (procedure \\texttt{AddAppliedEfforts()}), ":
f:=f."and listed below \\\\\n\n":
f:=f."\\footnotesize\n":
f:=f."\\verbatiminput{".filename.".AppEff.cpp}\n":
f:=f."\\normalsize\n\n":
end_if:
f:=f."\n\\section{Simulation}\n\n":
f:=f."The routine \\texttt{NewmarkIntegration} performs the integration of the equations of motion up to ":
f:=f."time \\emph{FinalTime} by regular time intervals equal to \\emph{StepSave}  and with the maximum allowed ":
f:=f."time step \\emph{StepMax} defined in the file \\texttt{".fileCPP."}. The following values are used:\n":
f:=f."\\begin{itemize}\n":
gen:=generate::TeX(FinalTime):
f:=f."   \\item \\emph{FinalTime} equal to ".gen." $s$,\n":
gen:=generate::TeX(StepSave):
f:=f."   \\item \\emph{StepSave} to ".gen." $s$,\n":
gen:=generate::TeX(StepMax):
f:=f."   \\item \\emph{StepMax} to ".gen." $s$.\n":
f:=f."\\end{itemize}\n":
f:=f."\\noindent The initial conditions are ":
for k from 0 to indexdof do
if qi[k]<>0 then
gen:=generate::TeX(qi[k]):
f:=f."$q_{".k."} = ".gen."$, ":
end_if:
if qdi[k]<>0 then
gen:=generate::TeX(qdi[k]):
f:=f."$\\dot{q}_{".k."} = ".gen."$, ":
end_if:
end_for:
f:=f."the others being equal to zero.\\\\\n":
f:=f."\n\\section{Results}\n\n":
f:=f."The time evolution of the different configuration parameters and their first and second derivatives ":
f:=f."can be easily plotted by \\textsf{Gnuplot}  as you can see in ":
f:=f."figures~\\ref{figure1} to~\\ref{figure3} etablished by the following script:\\\\\n\n":
f:=f."\\footnotesize\n":
f:=f."\\verbatiminput{".filename.".plt}\n":
f:=f."\\normalsize\n\n":
f:=f."\\begin{figure}[h!tb]\n":
f:=f."   \\begin{center}\n":
f:=f."      \\includegraphics[width=0.75\\textwidth]{figure1.eps}\n":
f:=f."   \\end{center}\n":
f:=f."   \\caption{Time evolution of parameters}\n":
f:=f."   \\label{figure1}\n":
f:=f."\\end{figure}\n\n":
f:=f."\\begin{figure}[h!tb]\n":
f:=f."   \\begin{center}\n":
f:=f."      \\includegraphics[width=0.75\\textwidth]{figure2.eps}\n":
f:=f."   \\end{center}\n":
f:=f."   \\caption{Time evolution of time derivatives of parameters}\n":
f:=f."   \\label{figure2}\n":
f:=f."\\end{figure}\n\n":
f:=f."\\begin{figure}[h!tb]\n":
f:=f."   \\begin{center}\n":
f:=f."      \\includegraphics[width=0.75\\textwidth]{figure3.eps}\n":
f:=f."   \\end{center}\n":
f:=f."   \\caption{Time evolution of second time derivatives of parameters}\n":
f:=f."   \\label{figure3}\n":
f:=f."\\end{figure}\n\n":
f:=f."\n\\appendix\n\n":
f:=f."\\clearpage\n":
f:=f."\\section{User's MuPAD code}\n":
f:=f."\\footnotesize\n":
f:=f."\\verbatiminput{".FichierUtil."}\n\n":
f:=f."\n\\end{document}\n":
fprint(Unquoted,Text,fileTEXEN,f):
timeTEXEN:=time()/1000:
if LATEX_FR=1 then ssTEXEN:=trunc(timeTEXEN-timeTEXFR):
else ssTEXEN:=trunc(timeTEXEN-timePLT):
end_if:
print(Unquoted,"... Writing in TEX file finished in ".ssTEXEN." second(s).\n"):
end_if:



// Total CPU-time.
print(Unquoted,"=========\n"):
timeTotal:=time()/1000:
ss:=timeTotal-time0:
mm:=trunc(ss/60):
seconds:=trunc(ss-mm*60):
print(Unquoted,"All the operations are finished. Total CPU-time in MuPAD: ".mm."min ".seconds."s.");



// Reset.
unexport(linalg):



// End of CAGeM procedure.
end_proc:
