// File in C++ format 
// Simulation of a simplified bike
 
 
//    copyright (C) 2003 Olivier VERLINDEN
//    Service de Mecanique rationnelle, Dynamique et Vibrations
//    Faculte Polytechnique de Mons
//    31, Bd Dolez, 7000 MONS (Belgium)
//    Olivier.Verlinden@fpms.ac.be
//
// This file is part of EasyDyn
//
// EasyDyn is free software; you can redistribute it and/or modify it under the
// terms of the GNU GeneralPublic License as published by the Free Software
// Foundation; either version 2, or (at your option) any later version.
//
// EasyDyn is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with 
// EasyDyn; see the file COPYING.  If not, write to the Free Software
// Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//


#define EASYDYNMBSMAIN  // to declare the global variable 
#include <math.h> 
#include <EasyDyn/mbs.h> 
#include <EasyDyn/visu.h> 
#include <EasyDyn/vec.h> 
#include <fstream> 

using namespace std ; 
scene thescene ; 
ofstream VanFile ; 



//---------------------------------------------------
void WriteDataHeader(ostream &OutFile) 
{
WriteStateVariablesHeader(OutFile) ;
OutFile << endl ;
}

//----------------------------------------------------
void SaveData(ostream &OutFile) 
{
SaveStateVariables(OutFile) ;
OutFile << endl ;
thescene.WriteCoord(VanFile) ; 
}

//----------------------------------------------------
void SetInertiaData() 
{
// Inertia for bodies
body[0].mass = 80.0 ;
body[1].mass = 2.0 ;
body[2].mass = 2.0 ;
body[0].PhiG.Ixx = 15.0 ;
body[0].PhiG.Iyy = 15.0 ;
body[0].PhiG.Izz = 0.6 ;
body[0].PhiG.Ixy = 0.0 ;
body[0].PhiG.Ixz = 0.0 ;
body[0].PhiG.Iyz = 0.0 ;
body[1].PhiG.Ixx = 0.06125 ;
body[1].PhiG.Iyy = 0.1225 ;
body[1].PhiG.Izz = 0.06125 ;
body[1].PhiG.Ixy = 0.0 ;
body[1].PhiG.Ixz = 0.0 ;
body[1].PhiG.Iyz = 0.0 ;
body[2].PhiG.Ixx = 0.06125 ;
body[2].PhiG.Iyy = 0.1225 ;
body[2].PhiG.Izz = 0.06125 ;
body[2].PhiG.Ixy = 0.0 ;
body[2].PhiG.Ixz = 0.0 ;
body[2].PhiG.Iyz = 0.0 ;
}

//----------------------------------------------------
void ComputeMotion() 
{
// Kinematics for bodies
body[0].T0G.R.r11 = cos(q[3])*cos(q[4]) ;
body[0].T0G.R.r12 = sin(q[3] + q[5])*(-1.0/2.0) - sin(q[3] - q[5])*(1.0/2.0) - cos(q[3] + q[4] + q[5])*(1.0/4.0) + cos(q[3] + q[4] - q[5])*(1.0/4.0) + cos(q[3] - q[4] + q[5])*(1.0/4.0) - cos(q[3] - q[4] - q[5])*(1.0/4.0) ;
body[0].T0G.R.r13 = cos(q[3] + q[5])*(-1.0/2.0) + cos(q[3] - q[5])*(1.0/2.0) + sin(q[3] + q[4] + q[5])*(1.0/4.0) + sin(q[3] + q[4] - q[5])*(1.0/4.0) - sin(q[3] - q[4] + q[5])*(1.0/4.0) - sin(q[3] - q[4] - q[5])*(1.0/4.0) ;
body[0].T0G.R.r21 = sin(q[3] + q[4])*(1.0/2.0) + sin(q[3] - q[4])*(1.0/2.0) ;
body[0].T0G.R.r22 = cos(q[3] + q[5])*(1.0/2.0) + cos(q[3] - q[5])*(1.0/2.0) - sin(q[3] + q[4] + q[5])*(1.0/4.0) + sin(q[3] + q[4] - q[5])*(1.0/4.0) + sin(q[3] - q[4] + q[5])*(1.0/4.0) - sin(q[3] - q[4] - q[5])*(1.0/4.0) ;
body[0].T0G.R.r23 = sin(q[3] + q[5])*(-1.0/2.0) + sin(q[3] - q[5])*(1.0/2.0) - cos(q[3] + q[4] + q[5])*(1.0/4.0) - cos(q[3] + q[4] - q[5])*(1.0/4.0) + cos(q[3] - q[4] + q[5])*(1.0/4.0) + cos(q[3] - q[4] - q[5])*(1.0/4.0) ;
body[0].T0G.R.r31 = -sin(q[4]) ;
body[0].T0G.R.r32 = sin(q[4] + q[5])*(1.0/2.0) - sin(q[4] - q[5])*(1.0/2.0) ;
body[0].T0G.R.r33 = cos(q[4])*cos(q[5]) ;
body[0].T0G.e.x = q[0] ;
body[0].T0G.e.y = q[1] ;
body[0].T0G.e.z = q[2] ;
body[1].TrefG.R.r11 = cos(q[6]) ;
body[1].TrefG.R.r12 = 0.0 ;
body[1].TrefG.R.r13 = sin(q[6]) ;
body[1].TrefG.R.r21 = 0.0 ;
body[1].TrefG.R.r22 = 1.0 ;
body[1].TrefG.R.r23 = 0.0 ;
body[1].TrefG.R.r31 = -sin(q[6]) ;
body[1].TrefG.R.r32 = 0.0 ;
body[1].TrefG.R.r33 = cos(q[6]) ;
body[1].TrefG.e.x = -1.0/2.0 ;
body[1].TrefG.e.y = 0.0 ;
body[1].TrefG.e.z = -0.5 ;
body[2].TrefG.R.r11 = sin(q[8])*0.2955202067 + cos(q[7] + q[8])*0.4776682446 + cos(q[7] - q[8]*1.0)*0.4776682446 ;
body[2].TrefG.R.r12 = sin(q[7])*-0.9553364891 ;
body[2].TrefG.R.r13 = cos(q[8])*-0.2955202067 + sin(q[7] + q[8])*0.4776682446 - sin(q[7] - q[8]*1.0)*0.4776682446 ;
body[2].TrefG.R.r21 = sin(q[7] + q[8])*(1.0/2.0) + sin(q[7] - q[8])*(1.0/2.0) ;
body[2].TrefG.R.r22 = cos(q[7]) ;
body[2].TrefG.R.r23 = cos(q[7] + q[8])*(-1.0/2.0) + cos(q[7] - q[8])*(1.0/2.0) ;
body[2].TrefG.R.r31 = sin(q[8])*-0.9553364891 + cos(q[7] + q[8])*0.1477601033 + cos(q[7] - q[8]*1.0)*0.1477601033 ;
body[2].TrefG.R.r32 = sin(q[7])*-0.2955202067 ;
body[2].TrefG.R.r33 = cos(q[8])*0.9553364891 + sin(q[7] + q[8])*0.1477601033 - sin(q[7] - q[8]*1.0)*0.1477601033 ;
body[2].TrefG.e.x = 1.0/2.0 ;
body[2].TrefG.e.y = 0.0 ;
body[2].TrefG.e.z = -0.5 ;
body[0].vG.x = qd[0] ;
body[0].vG.y = qd[1] ;
body[0].vG.z = qd[2] ;
body[1].vGrel.x = 0.0 ;
body[1].vGrel.y = 0.0 ;
body[1].vGrel.z = 0.0 ;
body[2].vGrel.x = 0.0 ;
body[2].vGrel.y = 0.0 ;
body[2].vGrel.z = 0.0 ;
body[0].omega.x = -(qd[4]*sin(q[3])) + (1.0/2.0)*qd[5]*cos(q[3] + q[4]) + (1.0/2.0)*qd[5]*cos(q[3] - q[4]) ;
body[0].omega.y = qd[4]*cos(q[3]) + (1.0/2.0)*qd[5]*sin(q[3] + q[4]) + (1.0/2.0)*qd[5]*sin(q[3] - q[4]) ;
body[0].omega.z = qd[3] - qd[5]*sin(q[4]) ;
body[1].omegarel.x = 0.0 ;
body[1].omegarel.y = qd[6] ;
body[1].omegarel.z = 0.0 ;
body[2].omegarel.x = qd[7]*-0.2955202067 - 0.9553364891*qd[8]*sin(q[7]) ;
body[2].omegarel.y = qd[8]*cos(q[7]) + 1.231778202e-12*qd[7]*sin(q[7]*2.0) ;
body[2].omegarel.z = qd[7]*0.9553364891 - 0.2955202067*qd[8]*sin(q[7]) ;
body[0].aG.x = qdd[0] ;
body[0].aG.y = qdd[1] ;
body[0].aG.z = qdd[2] ;
body[1].aGrel.x = 0.0 ;
body[1].aGrel.y = 0.0 ;
body[1].aGrel.z = 0.0 ;
body[2].aGrel.x = 0.0 ;
body[2].aGrel.y = 0.0 ;
body[2].aGrel.z = 0.0 ;
body[0].omegad.x = -(qdd[4]*sin(q[3])) - qd[3]*qd[4]*cos(q[3]) + (1.0/2.0)*qdd[5]*cos(q[3] + q[4]) + (1.0/2.0)*qdd[5]*cos(q[3] - q[4]) - (1.0/2.0)*qd[3]*qd[5]*sin(q[3] + q[4]) - (1.0/2.0)*qd[4]*qd[5]*sin(q[3] + q[4]) - (1.0/2.0)*qd[3]*qd[5]*sin(q[3] - q[4]) + (1.0/2.0)*qd[4]*qd[5]*sin(q[3] - q[4]) ;
body[0].omegad.y = qdd[4]*cos(q[3]) - qd[3]*qd[4]*sin(q[3]) + (1.0/2.0)*qdd[5]*sin(q[3] + q[4]) + (1.0/2.0)*qdd[5]*sin(q[3] - q[4]) + (1.0/2.0)*qd[3]*qd[5]*cos(q[3] + q[4]) + (1.0/2.0)*qd[4]*qd[5]*cos(q[3] + q[4]) + (1.0/2.0)*qd[3]*qd[5]*cos(q[3] - q[4]) - (1.0/2.0)*qd[4]*qd[5]*cos(q[3] - q[4]) ;
body[0].omegad.z = qdd[3] - qdd[5]*sin(q[4]) - qd[4]*qd[5]*cos(q[4]) ;
body[1].omegadrel.x = 0.0 ;
body[1].omegadrel.y = qdd[6] ;
body[1].omegadrel.z = 0.0 ;
body[2].omegadrel.x = qdd[7]*-0.2955202067 - 0.9553364891*qdd[8]*sin(q[7]) - 0.9553364891*qd[7]*qd[8]*cos(q[7]) ;
body[2].omegadrel.y = qdd[8]*cos(q[7]) - 1.0*qd[7]*qd[8]*sin(q[7]) + 1.231778202e-12*qdd[7]*sin(q[7]*2.0) + 2.463556404e-12*(qd[7]*qd[7])*cos(q[7]*2.0) ;
body[2].omegadrel.z = qdd[7]*0.9553364891 - 0.2955202067*qdd[8]*sin(q[7]) - 0.2955202067*qd[7]*qd[8]*cos(q[7]) ;
// Definition of relative motion
ComposeMotion(1,0);
ComposeMotion(2,0);
}

//----------------------------------------------------
void AddAppliedEfforts() 
{
// Contribution of gravity
vec gravity(0.0,0.0,-9.81) ;
AddGravityForces(gravity) ;
// Contribution of external forces
#include "bike3D.AppEff.cpp" 
}

//-----------------------------------------------------

void ComputeResidual()
{
ComputeResidualmbs();
}


//-----------------------------------------------------

int main()
{
// Initialisation and memory allocation
nbrdof = 9 ;
nbrbody = 3 ;
application = "bike3D" ;
InitEasyDynmbs() ;
// Let's create the shapes 
shape *s0 , *s1 , *s2 ; 
s0 = new box(&(body[0].T0G),vcoord(-0.1,-0.1,-0.1),vcoord(0.1,0.1,0.1),1,2) ; 
s1 = new box(&(body[1].T0G),vcoord(-0.1,-0.1,-0.1),vcoord(0.1,0.1,0.1),1,2) ; 
s2 = new box(&(body[2].T0G),vcoord(-0.1,-0.1,-0.1),vcoord(0.1,0.1,0.1),1,2) ; 
// Let's add the shapes to the scene 
thescene.AddShape(s0) ;
thescene.AddShape(s1) ;
thescene.AddShape(s2) ;
// Uncomment the following line if you want a moving observer 
// thescene.SetVisuFrame(&Tref) ; 
// Let's open an animation file 
VanFile.open("bike3D.van") ; 
// Initial configuration
q[0] = 0.0 ;
qd[0] = 5.0 ;
q[1] = 0.0 ;
qd[1] = 0.0 ;
q[2] = 0.85 ;
qd[2] = 0.0 ;
q[3] = 0.0 ;
qd[3] = 0.0 ;
q[4] = 0.0 ;
qd[4] = 0.0 ;
q[5] = 0.0 ;
qd[5] = 0.0 ;
q[6] = 0.0 ;
qd[6] = 14.28571429 ;
q[7] = 0.0 ;
qd[7] = 0.0 ;
q[8] = 0.0 ;
qd[8] = 14.28571429 ;
// Let's first search for the static equilibrium 
int *doflocked ; 
doflocked=new int[nbrdof] ; 
doflocked[0]=0 ; 
doflocked[1]=0 ; 
doflocked[2]=0 ; 
doflocked[3]=0 ; 
doflocked[4]=0 ; 
doflocked[5]=0 ; 
doflocked[6]=0 ; 
doflocked[7]=0 ; 
doflocked[8]=0 ; 
StaticEquilibrium(doflocked) ; 
// Let's save the structure of the scene 
ComputeMotion() ; 
thescene.CreateVolFile("bike3D.vol") ; 
// Let's go !
NewmarkIntegration(5.0,0.1,0.05) ;
// The clean way to finish 
EndEasyDynmbs() ;
VanFile.close() ; 
}

//-----------------------------------------------------
 

