// 

// Copyright (C) 2003 Olivier VERLINDEN
//     Service de Mecanique rationnelle, Dynamique et Vibrations
//     Faculte Polytechnique de Mons
//     31, Bd Dolez, 7000 MONS (Belgium)
//     Olivier.Verlinden@fpms.ac.be

// This file is part of EasyDyn

// EasyDyn is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation; either version 2, or (at your option) any
// later version.

// EasyDyn is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with EasyDyn; see the file COPYING.  If not, write to the Free
// Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//                            FACULTE POLYTECHNIQUE DE MONS
//
//               service de Mcanique Rationnelle, Dynamique et Vibrations
//
//
//                    -----------------------------------------------
//                                fichier utilisateur
//                             (crit en language MuPAD)
//
//
//                  Ir. Georges KOUROUSSIS - mars 2003
//
//
//

// Title of the application
title:="Simulation of a simplified bike":

// Definition of nbrdof  : Number of degrees of freedom
//               nbrbody : Number of bodies
//               nbrcont : Number of constraints (unused in this version).
nbrdof:= 9:
nbrbody:= 3:

// Gravity vector
gravity[1]:=0:
gravity[2]:=0:
gravity[3]:=-9.81:

// Eventual constants
L:=1:   // wheelbase
h:=0.5: // height of CG wrt to center of wheel
R1:=0.35: // outer radius of wheel
R2:=0.05: // half-width of tire
m0:=80:
mw:=2:
vini:=5:  // initial speed

// Inertia characteristics
mass[0]:=m0:
mass[1]:=mw:
mass[2]:=mw:
Ixx[0]:=m0*1.5*1.5/12:
Iyy[0]:=m0*1.5*1.5/12:
Izz[0]:=m0*0.3*0.3/12:
Ixx[1]:=mw*R1*R1/4:
Iyy[1]:=mw*R1*R1/2:
Izz[1]:=mw*R1*R1/4:
Ixx[2]:=mw*R1*R1/4:
Iyy[2]:=mw*R1*R1/2:
Izz[2]:=mw*R1*R1/4:

// Definition of the position matrices
T0G[0]:=Tdisp(q[0],q[1],q[2])*Trotz(q[3])*Troty(q[4])*Trotx(q[5]):
TrefG[1]:=Tdisp(-L/2,0,-h)*Troty(q[6]):
BodyRef[1]:=0:
TrefG[2]:=Tdisp(L/2,0,-h)*Troty(-0.3)*Trotz(q[7])*Troty(q[8]):
BodyRef[2]:=0:
// Initial conditions
qi[2]:=h+R1:
qdi[0]:=vini:
qdi[6]:=vini/R1:
qdi[8]:=vini/R1:

// Simulation parameters
FinalTime:=5:
StepSave:=0.1:
StepMax:=0.05:

SIMPLIFY:=1:
// Set FORCES to 1 in case you want to include *.AppEff.cpp into procedure
// AddAppliedEfforts() to define forces other than gravity
FORCES:=1:
// Set ANIM to 1 in case you want CaGEM to generate the skeleton code
// for visualization and animation of the system
ANIM:=1:
// Set STATIC to 1 in case you want CaGEM to generate the code
// to search for static equilibrium before integration
STATIC:=1:
// Set PLOT to 1 in case you want CaGEM to generate the GNUPLOT code
// to plot the evolution of position, velocity and acceeration
PLOT:=1:
// SET LATEX_FR to 1 if you want the LaTeX report in French
LATEX_FR:=0:
// SET LATEX_EN to 1 if you want the LaTeX report in English
LATEX_EN:=0:
