// File in C++ format 
// Simulation of a double pendulum
 
 
//    copyright (C) 2003 Olivier VERLINDEN
//    Service de Mecanique rationnelle, Dynamique et Vibrations
//    Faculte Polytechnique de Mons
//    31, Bd Dolez, 7000 MONS (Belgium)
//    Olivier.Verlinden@fpms.ac.be
//
// This file is part of EasyDyn
//
// EasyDyn is free software; you can redistribute it and/or modify it under the
// terms of the GNU GeneralPublic License as published by the Free Software
// Foundation; either version 2, or (at your option) any later version.
//
// EasyDyn is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with 
// EasyDyn; see the file COPYING.  If not, write to the Free Software
// Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//


#define EASYDYNMBSMAIN  // to declare the global variable 
#include <math.h> 
#include <EasyDyn/mbs.h> 
#include <EasyDyn/visu.h> 
#include <EasyDyn/vec.h> 
#include <fstream> 

using namespace std ; 
scene thescene ; 
ofstream VanFile ; 



//---------------------------------------------------
void WriteDataHeader(ostream &OutFile) 
{
WriteStateVariablesHeader(OutFile) ;
OutFile << endl ;
}

//----------------------------------------------------
void SaveData(ostream &OutFile) 
{
SaveStateVariables(OutFile) ;
OutFile << endl ;
thescene.WriteCoord(VanFile) ; 
}

//----------------------------------------------------
void SetInertiaData() 
{
// Inertia for body 0
body[0].mass = 1.1 ;
body[0].PhiG.Ixx = 1.0 ;
body[0].PhiG.Iyy = 1.0 ;
body[0].PhiG.Izz = 0.132 ;
body[0].PhiG.Ixy = 0.0 ;
body[0].PhiG.Ixz = 0.0 ;
body[0].PhiG.Iyz = 0.0 ;
// Inertia for body 1
body[1].mass = 0.9 ;
body[1].PhiG.Ixx = 1.0 ;
body[1].PhiG.Iyy = 1.0 ;
body[1].PhiG.Izz = 0.09075 ;
body[1].PhiG.Ixy = 0.0 ;
body[1].PhiG.Ixz = 0.0 ;
body[1].PhiG.Iyz = 0.0 ;
}

//----------------------------------------------------
void ComputeMotion() 
{
// Homogenous transformation matrices of each body
body[0].T0G.R.r11 = cos(q[0]) ;
body[0].T0G.R.r12 = -sin(q[0]) ;
body[0].T0G.R.r13 = 0.0 ;
body[0].T0G.R.r21 = sin(q[0]) ;
body[0].T0G.R.r22 = cos(q[0]) ;
body[0].T0G.R.r23 = 0.0 ;
body[0].T0G.R.r31 = 0.0 ;
body[0].T0G.R.r32 = 0.0 ;
body[0].T0G.R.r33 = 1.0 ;
body[0].T0G.e.x = sin(q[0])*0.6 ;
body[0].T0G.e.y = cos(q[0])*-0.6 ;
body[0].T0G.e.z = 0.0 ;
body[1].T0G.R.r11 = cos(q[0] + q[1]) ;
body[1].T0G.R.r12 = -sin(q[0] + q[1]) ;
body[1].T0G.R.r13 = 0.0 ;
body[1].T0G.R.r21 = sin(q[0] + q[1]) ;
body[1].T0G.R.r22 = cos(q[0] + q[1]) ;
body[1].T0G.R.r23 = 0.0 ;
body[1].T0G.R.r31 = 0.0 ;
body[1].T0G.R.r32 = 0.0 ;
body[1].T0G.R.r33 = 1.0 ;
body[1].T0G.e.x = sin(q[0])*1.2 + sin(q[0] + q[1])*0.55 ;
body[1].T0G.e.y = cos(q[0])*-1.2 - cos(q[0] + q[1])*0.55 ;
body[1].T0G.e.z = 0.0 ;
// Velocity vectors of the center of gravity of each body
body[0].vG.x = 0.6*qd[0]*cos(q[0]) ;
body[0].vG.y = 0.6*qd[0]*sin(q[0]) ;
body[0].vG.z = 0.0 ;
body[1].vG.x = 1.2*qd[0]*cos(q[0]) + 0.55*qd[0]*cos(q[0] + q[1]) + 0.55*qd[1]*cos(q[0] + q[1]) ;
body[1].vG.y = 1.2*qd[0]*sin(q[0]) + 0.55*qd[0]*sin(q[0] + q[1]) + 0.55*qd[1]*sin(q[0] + q[1]) ;
body[1].vG.z = 0.0 ;
// Rotational velocity vectors of each body
body[0].omega.x = 0.0 ;
body[0].omega.y = 0.0 ;
body[0].omega.z = qd[0] ;
body[1].omega.x = 0.0 ;
body[1].omega.y = 0.0 ;
body[1].omega.z = qd[0] + qd[1] ;
// Acceleration vectors of the center of gravity of each body
body[0].aG.x = 0.6*qdd[0]*cos(q[0]) - 0.6*(qd[0]*qd[0])*sin(q[0]) ;
body[0].aG.y = 0.6*qdd[0]*sin(q[0]) + 0.6*(qd[0]*qd[0])*cos(q[0]) ;
body[0].aG.z = 0.0 ;
body[1].aG.x = 1.2*qdd[0]*cos(q[0]) - 1.2*(qd[0]*qd[0])*sin(q[0]) + 0.55*qdd[0]*cos(q[0] + q[1]) + 0.55*qdd[1]*cos(q[0] + q[1]) - 0.55*(qd[0]*qd[0])*sin(q[0] + q[1]) - 0.55*(qd[1]*qd[1])*sin(q[0] + q[1]) - 1.1*qd[0]*qd[1]*sin(q[0] + q[1]) ;
body[1].aG.y = 1.2*qdd[0]*sin(q[0]) + 1.2*(qd[0]*qd[0])*cos(q[0]) + 0.55*qdd[0]*sin(q[0] + q[1]) + 0.55*qdd[1]*sin(q[0] + q[1]) + 0.55*(qd[0]*qd[0])*cos(q[0] + q[1]) + 0.55*(qd[1]*qd[1])*cos(q[0] + q[1]) + 1.1*qd[0]*qd[1]*cos(q[0] + q[1]) ;
body[1].aG.z = 0.0 ;
// Rotational acceleration vectors of each body
body[0].omegad.x = 0.0 ;
body[0].omegad.y = 0.0 ;
body[0].omegad.z = qdd[0] ;
body[1].omegad.x = 0.0 ;
body[1].omegad.y = 0.0 ;
body[1].omegad.z = qdd[0] + qdd[1] ;
}

//----------------------------------------------------
void AddAppliedEfforts() 
{
// Contribution of gravity
vec gravity(0.0,-9.81,0.0) ;
AddGravityForces(gravity) ;
}

//-----------------------------------------------------
void ComputeResidual()
{
ComputeResidualmbs();
}


//-----------------------------------------------------


int main()
{
// Initialisation and memory allocation
nbrdof = 2 ;
nbrbody = 2 ;
application = "dp3" ;
InitEasyDynmbs() ;
// Let's create the shapes 
shape *s0 , *s1 ; 
s0 = new box(&(body[0].T0G),vcoord(-0.1,-0.1,-0.1),vcoord(0.1,0.1,0.1),1,2) ; 
s1 = new box(&(body[1].T0G),vcoord(-0.1,-0.1,-0.1),vcoord(0.1,0.1,0.1),1,2) ; 
// Let's add the shapes to the scene 
thescene.AddShape(s0) ;
thescene.AddShape(s1) ;
// Uncomment the following line if you want a moving observer 
// thescene.SetVisuFrame(&Tref) ; 
// Let's open an animation file 
VanFile.open("dp3.van") ; 
// Initial configuration
q[0] = 0.0 ;
qd[0] = 0.0 ;
q[1] = 1.0 ;
qd[1] = 0.0 ;
// Let's save the structure of the scene 
ComputeMotion() ; 
thescene.CreateVolFile("dp3.vol") ; 
// Let's go !
NewmarkIntegration(5.0,0.01,0.005) ;
// The clean way to finish 
EndEasyDynmbs() ;
VanFile.close() ; 
}

//-----------------------------------------------------
 

