// File in C++ format 
// Simulation of a slider-crank mechanism
 
 
//    copyright (C) 2003 Olivier VERLINDEN
//    Service de Mecanique rationnelle, Dynamique et Vibrations
//    Faculte Polytechnique de Mons
//    31, Bd Dolez, 7000 MONS (Belgium)
//    Olivier.Verlinden@fpms.ac.be
//
// This file is part of EasyDyn
//
// EasyDyn is free software; you can redistribute it and/or modify it under the
// terms of the GNU GeneralPublic License as published by the Free Software
// Foundation; either version 2, or (at your option) any later version.
//
// EasyDyn is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with 
// EasyDyn; see the file COPYING.  If not, write to the Free Software
// Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//


#define EASYDYNMBSMAIN  // to declare the global variable 
#include <math.h> 
#include <EasyDyn/mbs.h> 
#include <EasyDyn/visu.h> 
#include <EasyDyn/vec.h> 
#include <fstream> 

using namespace std ; 


scene thescene ; 
ofstream VanFile ; 


//---------------------------------------------------
void WriteDataHeader(ostream &OutFile) 
{
WriteStateVariablesHeader(OutFile) ;
OutFile << endl ;
}

//----------------------------------------------------
void SaveData(ostream &OutFile) 
{
SaveStateVariables(OutFile) ;
OutFile << endl ;
thescene.WriteCoord(VanFile) ; 
}

//----------------------------------------------------
void SetInertiaData() 
{
// Inertia for bodies
body[0].mass = 1.0 ;
body[1].mass = 2.0 ;
body[2].mass = 5.0 ;
body[0].PhiG.Ixx = 1.0 ;
body[0].PhiG.Iyy = 1.0 ;
body[0].PhiG.Izz = 0.0833 ;
body[0].PhiG.Ixy = 0.0 ;
body[0].PhiG.Ixz = 0.0 ;
body[0].PhiG.Iyz = 0.0 ;
body[1].PhiG.Ixx = 1.0 ;
body[1].PhiG.Iyy = 1.0 ;
body[1].PhiG.Izz = 0.6667 ;
body[1].PhiG.Ixy = 0.0 ;
body[1].PhiG.Ixz = 0.0 ;
body[1].PhiG.Iyz = 0.0 ;
body[2].PhiG.Ixx = 1.0 ;
body[2].PhiG.Iyy = 1.0 ;
body[2].PhiG.Izz = 2.0 ;
body[2].PhiG.Ixy = 0.0 ;
body[2].PhiG.Ixz = 0.0 ;
body[2].PhiG.Iyz = 0.0 ;
}

//----------------------------------------------------
void ComputeMotion() 
{
// Kinematics for bodies
body[0].T0G.R.r11 = cos(q[0]) ;
body[0].T0G.R.r12 = -sin(q[0]) ;
body[0].T0G.R.r13 = 0.0 ;
body[0].T0G.R.r21 = sin(q[0]) ;
body[0].T0G.R.r22 = cos(q[0]) ;
body[0].T0G.R.r23 = 0.0 ;
body[0].T0G.R.r31 = 0.0 ;
body[0].T0G.R.r32 = 0.0 ;
body[0].T0G.R.r33 = 1.0 ;
body[0].T0G.e.x = cos(q[0])*(1.0/2.0) ;
body[0].T0G.e.y = sin(q[0])*(1.0/2.0) ;
body[0].T0G.e.z = 0.0 ;
body[1].T0G.R.r11 = pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) ;
body[1].T0G.R.r12 = sin(q[0])*(1.0/2.0) ;
body[1].T0G.R.r13 = 0.0 ;
body[1].T0G.R.r21 = sin(q[0])*(-1.0/2.0) ;
body[1].T0G.R.r22 = pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) ;
body[1].T0G.R.r23 = 0.0 ;
body[1].T0G.R.r31 = 0.0 ;
body[1].T0G.R.r32 = 0.0 ;
body[1].T0G.R.r33 = 1.0 ;
body[1].T0G.e.x = cos(q[0]) + pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) ;
body[1].T0G.e.y = sin(q[0])*(1.0/2.0) ;
body[1].T0G.e.z = 0.0 ;
body[2].T0G.R.r11 = 1.0 ;
body[2].T0G.R.r12 = 0.0 ;
body[2].T0G.R.r13 = 0.0 ;
body[2].T0G.R.r21 = 0.0 ;
body[2].T0G.R.r22 = 1.0 ;
body[2].T0G.R.r23 = 0.0 ;
body[2].T0G.R.r31 = 0.0 ;
body[2].T0G.R.r32 = 0.0 ;
body[2].T0G.R.r33 = 1.0 ;
body[2].T0G.e.x = cos(q[0]) + pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)*2.0 ;
body[2].T0G.e.y = 0.0 ;
body[2].T0G.e.z = 0.0 ;
body[0].vG.x = (-1.0/2.0)*qd[0]*sin(q[0]) ;
body[0].vG.y = (1.0/2.0)*qd[0]*cos(q[0]) ;
body[0].vG.z = 0.0 ;
body[1].vG.x = (1.0/4.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, -1.0/2.0)*((-1.0/2.0)*qd[0]*sin(q[0]*2.0) - 4.0*qd[0]*sin(q[0])*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)) ;
body[1].vG.y = (1.0/2.0)*qd[0]*cos(q[0]) ;
body[1].vG.z = 0.0 ;
body[2].vG.x = (1.0/2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, -1.0/2.0)*((-1.0/2.0)*qd[0]*sin(q[0]*2.0) - 2.0*qd[0]*sin(q[0])*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)) ;
body[2].vG.y = 0.0 ;
body[2].vG.z = 0.0 ;
body[0].omega.x = 0.0 ;
body[0].omega.y = 0.0 ;
body[0].omega.z = qd[0] ;
body[1].omega.x = 0.0 ;
body[1].omega.y = 0.0 ;
body[1].omega.z = (-1.0/2.0)*qd[0]*cos(q[0])*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, -1.0/2.0) ;
body[2].omega.x = 0.0 ;
body[2].omega.y = 0.0 ;
body[2].omega.z = 0.0 ;
body[0].aG.x = (-1.0/2.0)*qdd[0]*sin(q[0]) - (1.0/2.0)*(qd[0]*qd[0])*cos(q[0]) ;
body[0].aG.y = (1.0/2.0)*qdd[0]*cos(q[0]) - (1.0/2.0)*(qd[0]*qd[0])*sin(q[0]) ;
body[0].aG.z = 0.0 ;
body[1].aG.x = (1.0/16.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, -2.0)*((-85.0/8.0)*qdd[0]*sin(q[0]) - (27.0/16.0)*qdd[0]*sin(q[0]*3.0) - (113.0/8.0)*(qd[0]*qd[0])*cos(q[0]) - (1.0/16.0)*qdd[0]*sin(q[0]*5.0) - (29.0/16.0)*(qd[0]*qd[0])*cos(q[0]*3.0) - (1.0/16.0)*(qd[0]*qd[0])*cos(q[0]*5.0) - (1.0/8.0)*(qd[0]*qd[0])*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) + (1.0/8.0)*(qd[0]*qd[0])*cos(q[0]*4.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) + qdd[0]*sin(q[0]*2.0)*(pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)*(-7.0/4.0) - (1.0/4.0)*pow(cos(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) + (1.0/4.0)*pow(sin(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)) + (qd[0]*qd[0])*cos(q[0]*2.0)*(pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)*(-7.0/2.0) - (1.0/2.0)*pow(cos(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) + (1.0/2.0)*pow(sin(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0))) ;
body[1].aG.y = (1.0/2.0)*qdd[0]*cos(q[0]) - (1.0/2.0)*(qd[0]*qd[0])*sin(q[0]) ;
body[1].aG.z = 0.0 ;
body[2].aG.x = (1.0/8.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, -2.0)*((-85.0/16.0)*qdd[0]*sin(q[0]) - (27.0/32.0)*qdd[0]*sin(q[0]*3.0) - (113.0/16.0)*(qd[0]*qd[0])*cos(q[0]) - (1.0/32.0)*qdd[0]*sin(q[0]*5.0) - (29.0/32.0)*(qd[0]*qd[0])*cos(q[0]*3.0) - (1.0/32.0)*(qd[0]*qd[0])*cos(q[0]*5.0) - (1.0/8.0)*(qd[0]*qd[0])*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) + (1.0/8.0)*(qd[0]*qd[0])*cos(q[0]*4.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) + qdd[0]*sin(q[0]*2.0)*(pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)*(-7.0/4.0) - (1.0/4.0)*pow(cos(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) + (1.0/4.0)*pow(sin(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)) + (qd[0]*qd[0])*cos(q[0]*2.0)*(pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)*(-7.0/2.0) - (1.0/2.0)*pow(cos(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) + (1.0/2.0)*pow(sin(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0))) ;
body[2].aG.y = 0.0 ;
body[2].aG.z = 0.0 ;
body[0].omegad.x = 0.0 ;
body[0].omegad.y = 0.0 ;
body[0].omegad.z = qdd[0] ;
body[1].omegad.x = 0.0 ;
body[1].omegad.y = 0.0 ;
body[1].omegad.z = (1.0/32.0)*((-1.0/8.0)*(qd[0]*qd[0])*sin(q[0])*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) - (1.0/16.0)*(qd[0]*qd[0])*sin(q[0]*3.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) + (1.0/16.0)*(qd[0]*qd[0])*sin(q[0]*5.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) + qdd[0]*cos(q[0])*(pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)*(-7.0/8.0) - (1.0/8.0)*pow(cos(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) + (1.0/8.0)*pow(sin(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)) + (qd[0]*qd[0])*sin(q[0])*(pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)*(7.0/4.0) + (1.0/4.0)*pow(cos(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) - (1.0/4.0)*pow(sin(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)) + qdd[0]*cos(q[0]*3.0)*(pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)*(7.0/8.0) + (1.0/8.0)*pow(cos(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) - (1.0/8.0)*pow(sin(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)) + (qd[0]*qd[0])*sin(q[0]*3.0)*(pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)*(-7.0/4.0) - (1.0/4.0)*pow(cos(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) + (1.0/4.0)*pow(sin(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)) + qdd[0]*cos(q[0])*(pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)*(-49.0/4.0) - (7.0/2.0)*pow(cos(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) - (1.0/4.0)*pow(cos(q[0]), 4.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) + (7.0/2.0)*pow(sin(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) - (1.0/4.0)*pow(sin(q[0]), 4.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) + (1.0/2.0)*pow(cos(q[0]), 2.0)*pow(sin(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)) + (qd[0]*qd[0])*sin(q[0])*(pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)*(49.0/4.0) + (7.0/2.0)*pow(cos(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) + (1.0/4.0)*pow(cos(q[0]), 4.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) - (7.0/2.0)*pow(sin(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) + (1.0/4.0)*pow(sin(q[0]), 4.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0) - (1.0/2.0)*pow(cos(q[0]), 2.0)*pow(sin(q[0]), 2.0)*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, 1.0/2.0)))*pow(cos(q[0]*2.0)*(1.0/8.0) + 7.0/8.0, -2.0) ;
body[2].omegad.x = 0.0 ;
body[2].omegad.y = 0.0 ;
body[2].omegad.z = 0.0 ;
}

//----------------------------------------------------
void AddAppliedEfforts() 
{
// Contribution of gravity
vec gravity(0.0,-9.81,0.0) ;
AddGravityForces(gravity) ;
// Contribution of external forces
#include "slidercrank.AppEff.cpp" 
}

//-----------------------------------------------------

void ComputeResidual()

{
ComputeResidualmbs();
}

//--------------------------------------------------------------------

int main()
{
// Initialisation and memory allocation
nbrdof = 1 ;
nbrbody = 3 ;
application = "slidercrank" ;
InitEasyDynmbs() ;
// Let's create the shapes 
shape *s0 , *s1 , *s2 ; 
s0 = new box(&(body[0].T0G),vcoord(-0.5,-0.1,-0.1),vcoord(0.5,0.1,0.1),13,5) ; 
s1 = new box(&(body[1].T0G),vcoord(-1,-0.05,-0.05),vcoord(1,0.05,0.05),10,2) ; 
s2 = new frustum(&(body[2].T0G),vcoord(-0.25,0,0),vcoord(0.25,0,0),0.3,0.3,24,13,5) ; 
// Let's add the shapes to the scene 
thescene.AddShape(s0) ;
thescene.AddShape(s1) ;
thescene.AddShape(s2) ;
// Uncomment the following line if you want a moving observer 
// thescene.SetVisuFrame(&Tref) ; 
// Let's open an animation file 
VanFile.open("slidercrank.van") ; 
// Initial configuration
q[0] = 1.0 ;
qd[0] = 0.0 ;
// Let's save the structure of the scene 
ComputeMotion() ; 
thescene.CreateVolFile("slidercrank.vol") ; 
// Let's go !
NewmarkIntegration(6.0,0.05,0.01) ;
// The clean way to finish 
EndEasyDynmbs() ;
VanFile.close() ; 
}

//-----------------------------------------------------
 

