/*

Copyright (C) 2003 Olivier VERLINDEN
    Service de Mecanique rationnelle, Dynamique et Vibrations
    Faculte Polytechnique de Mons
    31, Bd Dolez, 7000 MONS (Belgium)
    Olivier.Verlinden@fpms.ac.be

This file is part of EasyDyn

EasyDyn is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

EasyDyn is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with EasyDyn; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include <EasyDyn/vec.h>
#include <EasyDyn/visu.h>

struct structbody
{ 
double mass;
tiner PhiG;
mth T0G,  // homogeneous transformation matrix giving the position 
            //of the body with respect to the global refernce frame
    TrefG; // idem for the relative position
vec vG,     // velocity of center of gravity 
    aG,     // acceleration of center of gravity 
    omega,  // rotation velocity of body
    omegad, // rotational acceleration of body
    // Relative motion terms, used in ComposeKinematics() and
    // assumed to be expressed in the coordinate system used as reference
    // for the relative motion
    vGrel,  // relative velocity of center of gravity 
    aGrel,  // relative acceleration of center of gravity 
    omegarel,  // relative rotation velocity of body
    omegadrel, // relative rotational acceleration of body
    // vectors used for forces
    R, MG,  // resultant force and moment at center of gravity
    vGsto, omegasto, // used for numerical derivation
    *vGpartial, *omegapartial,*vGrelpartial, *omegarelpartial;
int *underdof;
};

struct structtyre
{
double r1; // outer radius
double r2; // equivalent torus radius
double Kz; // vertical stiffness
double Cz; // vertical damping
double Fznom; // nominal vertical force on tyre
double Clongnom; // nominal longitudinal stiffness
double nlong; // Clong=Clongnom*(Fz/Fznom)^nlong
double Clatnom; // nominal cornering stiffness
double nlat; // Clat=Clatnom*(Fz/Fznom)^nlat
double Ccambernom; // nominal camber stiffness
double ncamber; // Ccamber=Ccambernom*(Fz/Fznom)^ncamber
double fClbs, fClbd; // Coulomb friction coefficients (static and dynamic)
};

#ifdef EASYDYNMBSMAIN
#define EASYDYNSIMMAIN
structbody *body=0;
double *p,*pd,*pdd;
int nbrbody=0,nbrdep=0;
int mbsAdvanced=0;
#else
extern structbody *body;
extern double *p,*pd,*pdd;
extern int nbrbody,nbrdep;
extern int mbsAdvanced;
#endif

#include <EasyDyn/sim.h>

//--------------------------------------------------------------------
// Procedures that must be provided by the user

void SetInertiaData();
void ComputeMotion();
void ComputePartialVelocities();
void AddAppliedEfforts();
void SaveData(ostream &OutFile); // eventually empty
void WriteDataHeader(ostream &OutFile); // eventually empty

//--------------------------------------------------------------------
// Procedures that we provide

void InitEasyDynmbs();
void EndEasyDynmbs();
void ComputeInertiaEfforts();
void ComputePartialVelocitiesDefault();
#ifdef EASYDYNMBSMAIN
#ifndef EASYDYNMBSADVANCED
void ComputePartialVelocities() { ComputePartialVelocitiesDefault(); }
#endif
#endif
void ComputeResidualmbs();
void ComposeMotion(int ibody, int ibodyref);
void ComposePartialVelocities(int ibody, int ibodyref);
void AddGravityForces(vec grav);
void AddSpringForce(double K, double L0, 
                    int ibodyA, vec rA, int ibodyB, vec rB);
void AddDamperForce(double C,
                    int ibodyA, vec rA, int ibodyB, vec rB);
void AddContactPlanePointForce(double Kn, double pk, 
                               double Cdamp, double pd,
                               double ffrict, double vglim,
                               int ibodyplane, vec rplane, vec nplane,
		               int ibodypoint, vec rpoint);
void AddTyreEfforts(int ibody, vec axe, structtyre tyre);

void CreateVmoFile(scene &sc);

//--------------------------------------------------------------------
